package org.openxma.dsl.pom.naming;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.xtext.linking.impl.LinkingHelper;
import org.eclipse.xtext.parsetree.AbstractNode;
import org.eclipse.xtext.parsetree.LeafNode;
import org.eclipse.xtext.parsetree.NodeUtil;
import org.eclipse.xtext.util.SimpleAttributeResolver;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.formatting.PomModelSerializer;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataMapping;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.ReferencedXMAPage;
import org.openxma.dsl.pom.model.XMAWidgetEventMapping;
import org.openxma.dsl.pom.model.XMAWidgetGrayLogic;
import org.openxma.dsl.pom.util.PomSwitch;

import com.google.common.base.Function;
import com.google.inject.Inject;

public class PomNameFunction extends PomSwitch<String> implements Function<EObject, String> {
	private SimpleAttributeResolver<EObject, String> resolver = SimpleAttributeResolver.newResolver(String.class, "name");

	@Inject
	private LinkingHelper linkingHelper;


	public String apply(EObject theEObject) {
		return doSwitch(theEObject);
	}

	@Override
	public String defaultCase(EObject object) {
		return resolver.apply(object);
	}

	
	@Override
	public String caseComponent(Component object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getComponent_XmaComponent());
	}

	@Override
	public String caseReferencedXMAPage(ReferencedXMAPage object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getReferencedXMAPage_XmaPage());
	}

	@Override
	public String caseReferencedXMAComposite(ReferencedXMAComposite object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getReferencedXMAComposite_XmaComposite());
	}

	@Override
	public String caseXMAWidgetGrayLogic(XMAWidgetGrayLogic object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getXMAWidgetGrayLogic_XmaWidget());
	}

	@Override
	public String caseXMAWidgetEventMapping(XMAWidgetEventMapping object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getXMAWidgetGrayLogic_XmaWidget());
	}

	@Override
	public String caseDataMapping(DataMapping object) {
		return getCrossReferenceAsString(object, PomPackage.eINSTANCE.getDataMapping_Control());
	}

	private String getCrossReferenceAsString(EObject object, EReference eReference) {
		List<AbstractNode> crossReferenceNodes = NodeUtil.findNodesForFeature(object, eReference);
		if (!crossReferenceNodes.isEmpty()) {
			LeafNode next = (LeafNode) crossReferenceNodes.iterator().next();
			String crossRefNodeAsString = null;
			if (linkingHelper != null) {
				crossRefNodeAsString = linkingHelper.getCrossRefNodeAsString(next, true);
			} else {
				crossRefNodeAsString = next.getText();
			}
			return crossRefNodeAsString;
		}
		return null;
	}

}
