package org.openxma.dsl.pom.validation;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.xtext.naming.IQualifiedNameProvider;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.validation.CoreDslJavaValidator;
import org.openxma.dsl.core.validation.ValidationCollectionUtil;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.PresentableFeature;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.AbsoluteSiblingAttachment;
import org.openxma.dsl.pom.model.AlignmentProperty;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.CustomizationOfComposite;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.HorizontalAlignment;
import org.openxma.dsl.pom.model.IField;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.Image;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.RelativeSibling;
import org.openxma.dsl.pom.model.RelativeSiblingAttachment;
import org.openxma.dsl.pom.model.SiblingAttachment;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.VerticalAlignment;
import org.openxma.dsl.pom.model.XMAWidgetEventMapping;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.scoping.PomDslScopeProvider;
import org.openxma.dsl.pom.util.LayoutData;

import at.spardat.xma.guidesign.XMAButton;
import at.spardat.xma.guidesign.XMATree;
import at.spardat.xma.guidesign.flex.IDefSelectFunctionCaller;
import at.spardat.xma.guidesign.flex.ISelectFunctionCaller;

import com.google.inject.Inject;

public class PomDslJavaValidator extends CoreDslJavaValidator {
	@Inject
	IQualifiedNameProvider qualifiedNameProvider;
	@Inject
	private PomDslScopeProvider scopeProvider;

	static String typesSupportedForComponentProperties[] = { "String", "Long", "Integer", "Double", "Date", "BigDecimal",
			"Boolean", "Float" };
	static String typesSupportedForPageProperties[] = { "String", "Long", "Integer", "Double", "Date", "BigDecimal", "Float" };

	public static String[] getTypesSupportedForComponentProperties() {
		return typesSupportedForComponentProperties;
	}

	@Override
	protected List<EPackage> getEPackages() {
		List<EPackage> result = new ArrayList<EPackage>();
		result.add(org.openxma.dsl.core.CorePackage.eINSTANCE);
		result.add(org.openxma.dsl.dom.DomPackage.eINSTANCE);
		result.add(org.openxma.dsl.pom.PomPackage.eINSTANCE);
		return result;
	}

	@Check(CheckType.FAST)
	public void checkEventTypeAllowed(final EventMapping eventMapping) {
		if (eventMapping instanceof GuiElementEventMapping) {
			GuiElementEventMapping ge = (GuiElementEventMapping) eventMapping;
			IGuiElementWithEvent control = ge.getControl();
			EventType et = ge.getEvent();
			boolean warning = false;
			// No check on NULL
			if (et.getValue() == EventType.NULL_VALUE)
				return;
			// Button
			if (control instanceof Button) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					warning = true;
			}
			// Table
			if (control instanceof Table) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					return;
				if (et.getValue() == EventType.ON_DOUBLE_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_DEFAULT_SELECTION_VALUE)
					warning = true;
			}
			// Combo
			if (control instanceof Combo) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					return;
			}
			// Tree
			if (control instanceof Tree) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					return;
				if (et.getValue() == EventType.ON_DOUBLE_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_DEFAULT_SELECTION_VALUE)
					warning = true;
				if (et.getValue() == EventType.ON_EXPAND_VALUE)
					return;
				if (et.getValue() == EventType.ON_COLLAPSE_VALUE)
					return;
			}
			// CustomizeableField
			if (control instanceof CustomizeableField) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					warning = true;
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					return;
			}
			// Image
			if (control instanceof Image) {
				if (et.getValue() == EventType.ON_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_SELECTION_VALUE)
					warning = true;
				if (et.getValue() == EventType.ON_DOUBLE_CLICK_VALUE)
					return;
				if (et.getValue() == EventType.ON_DEFAULT_SELECTION_VALUE)
					warning = true;
			}
			if (warning) {
				warning("The event " + et.getName() + " is deprecated for " + control.getName(), null);
			} else {
				error("The event " + et.getName() + " is not available for " + control.getName(), null);
			}
		}
	}

	@Check(CheckType.FAST)
	public void checkPropertyTypeAllowed(final ObjectProperty objectProperty) {
		if (objectProperty.getName() == null)
			return;
		String name = objectProperty.getType().getName();
		if (objectProperty.eContainer() instanceof Page) {
			for (int i = 0; i < typesSupportedForPageProperties.length; i++) {
				if (typesSupportedForPageProperties[i].equals(name))
					return;
			}
			error("The type '" + name + "' is not supported for a page property", null);
		} else {
			for (int i = 0; i < typesSupportedForComponentProperties.length; i++) {
				if (typesSupportedForComponentProperties[i].equals(name))
					return;
			}
			error("The type '" + name + "' is not supported for a component property", null);
		}
	}

	@Check(CheckType.FAST)
	public void checkCustomizationOfCompositeAllowed(final CustomizationOfComposite customizationOfComposite) {
		if (customizationOfComposite.getComposite() instanceof XmadslComposite) {
			XmadslComposite xmadslComposite = (XmadslComposite) customizationOfComposite.getComposite();
			// Base composite is a vpanel
			if (xmadslComposite.isHorizontalPartitioning() && customizationOfComposite.isHorizontalPartitioning() == false) {
				error("Panel types does not match: 'hpanel' can only be extended by 'hpanel'", null);
				return;
			}
			// Base composite is a hpanel
			if (xmadslComposite.isVerticalPartitioning() && customizationOfComposite.isVerticalPartitioning() == false) {
				error("Panel types does not match: 'vpanel' can only be extended by 'vpanel'", null);
				return;
			}
			// Base composite is a panel
			if (xmadslComposite.isHorizontalPartitioning() == false
					&& xmadslComposite.isVerticalPartitioning() == false
					&& (customizationOfComposite.isVerticalPartitioning() || customizationOfComposite
							.isHorizontalPartitioning())) {
				error("Panel types does not match: 'panel' can only be extended by 'panel'", null);
				return;
			}
		}
	}

	@Check(CheckType.FAST)
	public void checkCellSizeSpecification(final ComposeData composeData) {
		XmadslComposite xmadslComposite = null;
		if (composeData.eContainer() instanceof Content) {
			Content content = (Content) composeData.eContainer();
			if (content.eContainer() instanceof XmadslComposite) {
				xmadslComposite = (XmadslComposite) content.eContainer();
			}
		}
		if (composeData.getCellWidths().size() > 0) {
			if (xmadslComposite != null) {
				for (LayoutDataProperty layoutProp : xmadslComposite.getLayoutProperties()) {
					if (layoutProp instanceof AlignmentProperty) {
						AlignmentProperty alignmentProp = (AlignmentProperty) layoutProp;
						if (alignmentProp.getHAlignment().getValue() == HorizontalAlignment.RIGHT_VALUE) {
							error("Cell widths can not be defined for right aligned hpanels", null);
							break;
						}
					}
				}
			}
			if (xmadslComposite == null || xmadslComposite.isHorizontalPartitioning() == false) {
				error("Cell widths can only be defined for hpanel", null);
			}
			int strechCount = 0;
			for (TabulatorPosition pos : composeData.getCellWidths()) {
				if (pos.isFillingPosition())
					strechCount++;
			}
			if (strechCount > 1) {
				error("Only one cell can be stretched horizontally", null);
			}
		}
		if (composeData.getCellHeights().size() > 0) {
			if (xmadslComposite != null) {
				for (LayoutDataProperty layoutProp : xmadslComposite.getLayoutProperties()) {
					if (layoutProp instanceof AlignmentProperty) {
						AlignmentProperty alignmentProp = (AlignmentProperty) layoutProp;
						if (alignmentProp.getVAlignment().getValue() == VerticalAlignment.BOTTOM_VALUE) {
							error("Cell widths can not be defined for bottom aligned vpanels", null);
							break;
						}
					}
				}
			}
			if (xmadslComposite == null || xmadslComposite.isVerticalPartitioning() == false) {
				error("Cell heights can only be defined for vpanel", null);
			}
			int strechCount = 0;
			for (TabulatorPosition pos : composeData.getCellHeights()) {
				if (pos.isFillingPosition())
					strechCount++;
			}
			if (strechCount > 1) {
				error("Only one cell can be stretched vertically", null);
			}
		}
	}

	public static FieldPartSpecification getFieldPartSpecificationContainer(final SiblingAttachment attachment) {
		if (attachment.eContainer() instanceof AttachmentProperty) {
			AttachmentProperty attachmentProperty = (AttachmentProperty) attachment.eContainer();
			if (attachmentProperty.eContainer() instanceof FieldPartSpecification) {
				FieldPartSpecification fps = (FieldPartSpecification) attachmentProperty.eContainer();
				return fps;
			}
		}
		return null;
	}

	@Check(CheckType.NORMAL)
	public void checkRelativeAttachmentAllowed(final SiblingAttachment siblingAttachment) {

		if (siblingAttachment instanceof AbsoluteSiblingAttachment) {
			AbsoluteSiblingAttachment asa = (AbsoluteSiblingAttachment) siblingAttachment;
			if (asa.getSibling() == null && asa.getFieldPart() != null) {
				FieldPartSpecification fps = getFieldPartSpecificationContainer(siblingAttachment);
				if (fps == null) {
					error("Attachment to field parts can only used within a field part specivication", null);
					return;
				}
				if (fps.eContainer() instanceof StyleSpecificationProperty)
					return;
				CustomizeableField f = (CustomizeableField) fps.eContainer();
				FieldPart fp = asa.getFieldPart();
				for (FieldPartSpecification afps : f.getParts()) {
					if (afps.getPart().getValue() == fp.getValue()) {
						return;
					}
				}
				error("The referenced field part is not defined for this field", null);
				return;
			}
		}
		// This additional check necessary: siblingAttachment.getSibling() ==
		// null
		// because sometimes relSibling is not null but absolute sibling is
		// modeled
		// if (relSibling != null && siblingAttachment.getSibling() == null) {
		if (siblingAttachment instanceof RelativeSiblingAttachment) {
			RelativeSiblingAttachment rsa = (RelativeSiblingAttachment) siblingAttachment;

			RelativeSibling relSibling = rsa.getRelativeSibling();

			LayoutData layoutData = PomDslScopeProvider.getLayoutDataOfSiblingAttachment(siblingAttachment);
			GuiElement guiElement = (GuiElement) layoutData.getElementWithLayoutData();

			EObject containerOfSiblings = PomDslScopeProvider.getContainerOfSiblings(guiElement);
			List<EObject> siblings = PomDslScopeProvider.getContainedGuiElements(containerOfSiblings);

			int index = siblings.indexOf(guiElement);
			if (index == -1) {
				throw new RuntimeException("Element ( " + guiElement + " )not contained by expected container");
			}

			if (relSibling.getValue() == RelativeSibling.NEXT_VALUE) {
				if (index == siblings.size() - 1) {
					error("Next element not defined", null);
					return;
				}
			} else if (relSibling.getValue() == RelativeSibling.PREVIOUS_VALUE) {
				if (index == 0) {
					error("Previous element not defined", null);
					return;
				}
			}
		}
	}

	@Check(CheckType.NORMAL)
	public void checkGuiElementUnique(final GuiElement guiElement) {
		final String guiElementId = qualifiedNameProvider.apply(guiElement);
		if (guiElementId != null) {
			EObject nsContainer = scopeProvider.getNsContainer(guiElement);
			List<EObject> nsElements = scopeProvider.getNsElements(nsContainer);
			if (ValidationCollectionUtil.hasDuplicateElementsWithId(nsElements, guiElementId, qualifiedNameProvider)) {
				error(
						"Duplicate page element detected " + guiElement.getClass().getSimpleName() + ": " + guiElementId
								+ "'", null);
			}
		}
	}

	@Check(CheckType.NORMAL)
	public void checkCompositeSimpleLayout(ReferencedXMAComposite composite) {
		boolean check = composite instanceof XmadslComposite || composite.getXmaComposite().isYnSimpleLayout();
		if (!check) {
			error("Composite '" + qualifiedNameProvider.apply(composite) + "' has no SimpleLayout and can't be modelled",
					null);
		}
	}

	@Check(CheckType.NORMAL)
	public void checkField(IField field) {
		ComplexType parent = field.getObject().getType();
		if (field.getFeature() == null) {
			error("There is no feature specified", PomPackage.IFIELD__FEATURE);
			return;
		}
		if (isFeatureValid(parent, field.getFeature()))
			return;

		/*
		 * error("Attribute '" +
		 * qualifiedNameProvider.apply(field.getAttributeHolder()) +
		 * "' is not a member of '" +
		 * qualifiedNameProvider.apply(field.getObject().getType()) + "'",
		 * PomPackage.IFIELD__FEATURE);
		 */

		// if
		// (!field.getObject().getType().getAllAttributeHolders().contains(field.getAttributeHolder()))
		// {
		// error("Attribute '" + ID_FUNCTION.apply(field.getAttributeHolder()) +
		// "' is not a member of '"
		// + ID_FUNCTION.apply(field.getObject().getType()) + "'",
		// PomPackage.IFIELD__FEATURE);
		// }
	}

	public static Type getTypeOfPresentableFeature(PresentableFeature f) {
		Type type = null;
		if (f.getAttribute() != null) {
			Attribute a = f.getAttribute();
			type = a.getType().getDataType();
		}
		return type;
	}

	private boolean isFeatureValid(ComplexType parent, FieldFeature fieldFeature) {

		PresentableFeature f = fieldFeature.getFeature();

		if (f.getAttribute() != null) {
			Attribute a = f.getAttribute();
			if (parent.getAllPresentableFeaturesIncludingSuperType().contains(f) == false) {
				error("Attribute '" + qualifiedNameProvider.apply(a) + "' is not a member of '"
						+ qualifiedNameProvider.apply(parent) + "'", PomPackage.IFIELD__FEATURE);
				return false;
			}
		}

		Type type = getTypeOfPresentableFeature(f);
		if (type instanceof ComplexType) {
			if (fieldFeature.getSubFeature() != null) {
				return isFeatureValid((ComplexType) type, fieldFeature.getSubFeature());
			}
			error("The feature " + type.getName() + " is complex and therefore must habe a subfeature specified",
					PomPackage.IFIELD__FEATURE);
			return false;
		}
		return true;

		// if (f instanceof AttributeHolder) {
		// Attribute a = ((AttributeHolder)f).getAttribute();
		// if (a.getType().getDataType() instanceof SimpleType) {
		// return;
		// }
		// if (a.getType().getDataType() instanceof ValueObject) {
		// parent = (ValueObject)a.getType().getDataType();
		// continue;
		// }
		// throw new RuntimeException("Unknown type of AttributeHolder: "+f);
		// }
		// if (f instanceof Reference) {
		// parent = ((Reference)f).getType();
		// continue;
		// }

		// return false;
	}

	@Check(CheckType.NORMAL)
	public void checkControl(XMAWidgetEventMapping widgetEventMapping) {
		if (!(widgetEventMapping.getControl() instanceof ISelectFunctionCaller)
				&& !(widgetEventMapping.getControl() instanceof XMATree)) {
			error(
					"Type of widget '" + qualifiedNameProvider.apply(widgetEventMapping)
							+ "' does not support event mapping!", PomPackage.XMA_WIDGET_EVENT_MAPPING__CONTROL);
		}
	}

	@Check(CheckType.NORMAL)
	public void checkEventType(XMAWidgetEventMapping widgetEventMapping) {
		boolean check = false;
		switch (widgetEventMapping.getEvent()) {
		case ON_SELECTION:
			check = widgetEventMapping.getControl() instanceof ISelectFunctionCaller;
			break;
		case ON_DEFAULT_SELECTION:
			check = widgetEventMapping.getControl() instanceof IDefSelectFunctionCaller;
			break;
		case ON_EXPAND:
			check = widgetEventMapping.getControl() instanceof XMATree;
			break;
		case ON_COLLAPSE:
			check = widgetEventMapping.getControl() instanceof XMATree;
			break;
		case NULL:
			check = widgetEventMapping.getControl() instanceof XMAButton;
			break;
		default:
			check = false;
		}
		if (!check) {
			error("Widget '" + qualifiedNameProvider.apply(widgetEventMapping.getControl()) + "' does not support event '"
					+ widgetEventMapping.getEvent().getName() + "'", PomPackage.XMA_WIDGET_EVENT_MAPPING__EVENT);
		}
	}

}
