/**
 * <copyright>
 * </copyright>
 *
 * $Id: PomPackageImpl.java 6662 2010-09-27 19:49:43Z hoenninger $
 */
package org.openxma.dsl.pom.model.impl;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.impl.EPackageImpl;
import org.openxma.dsl.core.CorePackage;
import org.openxma.dsl.dom.DomPackage;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.AbsoluteSiblingAttachment;
import org.openxma.dsl.pom.model.AlignmentProperty;
import org.openxma.dsl.pom.model.AttachmentOwnPosition;
import org.openxma.dsl.pom.model.AttachmentPosition;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.AttachmentSiblingPosition;
import org.openxma.dsl.pom.model.AttachmentSpecification;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.CollapsedFlag;
import org.openxma.dsl.pom.model.CombinedModelElement;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.ComplexElement;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.ComposedElement;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.ContentAlignment;
import org.openxma.dsl.pom.model.ControlEventMapping;
import org.openxma.dsl.pom.model.ControlType;
import org.openxma.dsl.pom.model.CustomizationOfComposite;
import org.openxma.dsl.pom.model.CustomizationOfGuiElement;
import org.openxma.dsl.pom.model.CustomizationOfTabFolder;
import org.openxma.dsl.pom.model.CustomizeAttributeList;
import org.openxma.dsl.pom.model.CustomizeComponentModel;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.CustomizedAttribute;
import org.openxma.dsl.pom.model.DataBindingElement;
import org.openxma.dsl.pom.model.DataMapping;
import org.openxma.dsl.pom.model.DataMappingList;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.Definitions;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.EnterEventMapping;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.FieldReference;
import org.openxma.dsl.pom.model.FieldVariable;
import org.openxma.dsl.pom.model.GrayLogic;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.GuiElementWithEvent_dummy;
import org.openxma.dsl.pom.model.HeightProperty;
import org.openxma.dsl.pom.model.HorizontalAlignment;
import org.openxma.dsl.pom.model.IAttachmentPosition;
import org.openxma.dsl.pom.model.IComposite;
import org.openxma.dsl.pom.model.IDummy;
import org.openxma.dsl.pom.model.IElementOnWhichCanBeAttached;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.IField;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.ITabFolder;
import org.openxma.dsl.pom.model.ITabPage;
import org.openxma.dsl.pom.model.Image;
import org.openxma.dsl.pom.model.ImageUri;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.IntegerDefinition;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LabelText;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.LeaveEventMapping;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.MandatoryFlag;
import org.openxma.dsl.pom.model.ModelElementSpecification;
import org.openxma.dsl.pom.model.NoneAttachment;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.Offset;
import org.openxma.dsl.pom.model.Orientation;
import org.openxma.dsl.pom.model.PaddingWidth;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.PageCustomization;
import org.openxma.dsl.pom.model.PageDefinition;
import org.openxma.dsl.pom.model.PageProperty;
import org.openxma.dsl.pom.model.ParentAttachment;
import org.openxma.dsl.pom.model.PredefinedCommand;
import org.openxma.dsl.pom.model.RadioButton;
import org.openxma.dsl.pom.model.ReferencedComposite;
import org.openxma.dsl.pom.model.ReferencedTabFolder;
import org.openxma.dsl.pom.model.ReferencedTabPage;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.ReferencedXMAGuiElement;
import org.openxma.dsl.pom.model.ReferencedXMAPage;
import org.openxma.dsl.pom.model.RelativeSibling;
import org.openxma.dsl.pom.model.RelativeSiblingAttachment;
import org.openxma.dsl.pom.model.ResizeableProperty;
import org.openxma.dsl.pom.model.SelectedModelElement;
import org.openxma.dsl.pom.model.Seperator;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.SiblingAttachment;
import org.openxma.dsl.pom.model.SimpleElement;
import org.openxma.dsl.pom.model.StringDefinition;
import org.openxma.dsl.pom.model.StyleProperty;
import org.openxma.dsl.pom.model.StylePropertyBackgroundColor;
import org.openxma.dsl.pom.model.StylePropertyBorder;
import org.openxma.dsl.pom.model.StylePropertyBottom;
import org.openxma.dsl.pom.model.StylePropertyFieldPart;
import org.openxma.dsl.pom.model.StylePropertyForegroundColor;
import org.openxma.dsl.pom.model.StylePropertyHeight;
import org.openxma.dsl.pom.model.StylePropertyLeft;
import org.openxma.dsl.pom.model.StylePropertyRight;
import org.openxma.dsl.pom.model.StylePropertyTabulator;
import org.openxma.dsl.pom.model.StylePropertyTop;
import org.openxma.dsl.pom.model.StylePropertyWidgetVariant;
import org.openxma.dsl.pom.model.StylePropertyWidth;
import org.openxma.dsl.pom.model.StyleSelectorType;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.TabulatorAttachment;
import org.openxma.dsl.pom.model.TabulatorDefinition;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.TextProperties;
import org.openxma.dsl.pom.model.TitleButton;
import org.openxma.dsl.pom.model.TitleButtonsProperty;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.VerticalAlignment;
import org.openxma.dsl.pom.model.VisibleFlag;
import org.openxma.dsl.pom.model.WidthProperty;
import org.openxma.dsl.pom.model.XMAWidgetEventMapping;
import org.openxma.dsl.pom.model.XMAWidgetGrayLogic;
import org.openxma.dsl.pom.model.XmaVariable;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;
import org.openxma.dsl.pom.model.*;

import at.spardat.xma.guidesign.GuidesignPackage;
import at.spardat.xma.guidesign.flex.FlexPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model <b>Package</b>.
 * <!-- end-user-doc -->
 * @generated
 */
public class PomPackageImpl extends EPackageImpl implements PomPackage {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataBindingElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass enabledFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass mandatoryFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass visibleFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass collapsedFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass editableFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass labelTextEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass objectPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass componentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMAGuiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMAPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMACompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass grayLogicEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaWidgetGrayLogicEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataObjectVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizeAttributeListEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizedAttributeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass textPropertiesEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iFieldEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldFeatureEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldReferenceEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizeableFieldEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldPartSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass commandEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventFunctionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventMappingListEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass controlEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaWidgetEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass initEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass enterEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass leaveEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataMappingListEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmadslPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageCustomizationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass compositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass composedElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass complexElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass simpleElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iDummyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iGuiElementWithEventEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementWithEvent_dummyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass textEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass comboEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass checkBoxEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass listBoxEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass treeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass radioButtonEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass seperatorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass labelEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass imageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass buttonEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmadslCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tableColumnEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iTabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedTabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass layoutDataPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pagePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass heightPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass widthPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass titleButtonsPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass resizeablePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass alignmentPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass composeDataEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass paddingWidthEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iAttachmentPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stringDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass integerDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass imageUriEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass definitionsEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass contentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass setOfGuiElementsEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass noneAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass parentAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iElementOnWhichCanBeAttachedEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass siblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass absoluteSiblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass relativeSiblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass offsetEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizeComponentModelEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfGuiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iElementWithLayoutDataEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass styleSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass modelElementSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass selectedModelElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass combinedModelElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass styleSpecificationPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyFieldPartEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyLeftEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyRightEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTopEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBottomEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyWidthEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyHeightEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTabulatorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBorderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyForegroundColorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBackgroundColorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyWidgetVariantEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum eventTypeEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum predefinedCommandEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum attachmentOwnPositionEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum relativeSiblingEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum attachmentSiblingPositionEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum contentAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum horizontalAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum verticalAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum orientationEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum controlTypeEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum fieldPartEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum titleButtonEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum styleSelectorTypeEEnum = null;

	/**
	 * Creates an instance of the model <b>Package</b>, registered with
	 * {@link org.eclipse.emf.ecore.EPackage.Registry EPackage.Registry} by the package
	 * package URI value.
	 * <p>Note: the correct way to create the package is via the static
	 * factory method {@link #init init()}, which also performs
	 * initialization of the package, or returns the registered package,
	 * if one already exists.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see org.eclipse.emf.ecore.EPackage.Registry
	 * @see org.openxma.dsl.pom.PomPackage#eNS_URI
	 * @see #init()
	 * @generated
	 */
	private PomPackageImpl() {
		super(eNS_URI, PomFactory.eINSTANCE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private static boolean isInited = false;

	/**
	 * Creates, registers, and initializes the <b>Package</b> for this model, and for any others upon which it depends.
	 * 
	 * <p>This method is used to initialize {@link PomPackage#eINSTANCE} when that field is accessed.
	 * Clients should not invoke it directly. Instead, they should simply access that field to obtain the package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #eNS_URI
	 * @see #createPackageContents()
	 * @see #initializePackageContents()
	 * @generated
	 */
	public static PomPackage init() {
		if (isInited) return (PomPackage)EPackage.Registry.INSTANCE.getEPackage(PomPackage.eNS_URI);

		// Obtain or create and register package
		PomPackageImpl thePomPackage = (PomPackageImpl)(EPackage.Registry.INSTANCE.get(eNS_URI) instanceof PomPackageImpl ? EPackage.Registry.INSTANCE.get(eNS_URI) : new PomPackageImpl());

		isInited = true;

		// Initialize simple dependencies
		DomPackage.eINSTANCE.eClass();
		GuidesignPackage.eINSTANCE.eClass();

		// Create package meta-data objects
		thePomPackage.createPackageContents();

		// Initialize created meta-data
		thePomPackage.initializePackageContents();

		// Mark meta-data to indicate it can't be changed
		thePomPackage.freeze();

  
		// Update the registry and return the package
		EPackage.Registry.INSTANCE.put(PomPackage.eNS_URI, thePomPackage);
		return thePomPackage;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataBindingElement() {
		return dataBindingElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldFlag() {
		return fieldFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldFlag_Expression() {
		return (EReference)fieldFlagEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEnabledFlag() {
		return enabledFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getMandatoryFlag() {
		return mandatoryFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getVisibleFlag() {
		return visibleFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCollapsedFlag() {
		return collapsedFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEditableFlag() {
		return editableFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLabelText() {
		return labelTextEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabelText_Label() {
		return (EAttribute)labelTextEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabelText_ReferencedLabelString() {
		return (EReference)labelTextEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getObjectProperty() {
		return objectPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getObjectProperty_Public() {
		return (EAttribute)objectPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_Type() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getObjectProperty_Name() {
		return (EAttribute)objectPropertyEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_ReadOnly() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_Required() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComponent() {
		return componentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_XmaComponent() {
		return (EReference)componentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_StyleProperty() {
		return (EReference)componentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Dependencies() {
		return (EReference)componentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_ComponentProperties() {
		return (EReference)componentEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Dataobjects() {
		return (EReference)componentEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_CustomizedAttributeList() {
		return (EReference)componentEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Commands() {
		return (EReference)componentEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Events() {
		return (EReference)componentEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_ConditionsBlock() {
		return (EReference)componentEClass.getEStructuralFeatures().get(8);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Pages() {
		return (EReference)componentEClass.getEStructuralFeatures().get(9);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Definitions() {
		return (EReference)componentEClass.getEStructuralFeatures().get(10);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAGuiElement() {
		return referencedXMAGuiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAPage() {
		return referencedXMAPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_XmaPage() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_FieldFlags() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_Dependencies() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_GrayLogic() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAComposite() {
		return referencedXMACompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_ReferencingPageComposite() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_VerticalPartitioning() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_HorizontalPartitioning() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAComposite_XmaComposite() {
		return (EReference)referencedXMACompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAComposite_FieldFlags() {
		return (EReference)referencedXMACompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGrayLogic() {
		return grayLogicEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getGrayLogic_WidgetGrayLogic() {
		return (EReference)grayLogicEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXMAWidgetGrayLogic() {
		return xmaWidgetGrayLogicEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetGrayLogic_XmaWidget() {
		return (EReference)xmaWidgetGrayLogicEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetGrayLogic_FieldFlags() {
		return (EReference)xmaWidgetGrayLogicEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataObjectVariable() {
		return dataObjectVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataObjectVariable_Type() {
		return (EReference)dataObjectVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getDataObjectVariable_IsCollection() {
		return (EAttribute)dataObjectVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getDataObjectVariable_Name() {
		return (EAttribute)dataObjectVariableEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataObjectVariable_CustomizedAttributes() {
		return (EReference)dataObjectVariableEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizeAttributeList() {
		return customizeAttributeListEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeAttributeList_CustomizedAttributes() {
		return (EReference)customizeAttributeListEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizedAttribute() {
		return customizedAttributeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_Field() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_Constraints() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_Format() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_AttributFlag() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_AttributFlags() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizedAttribute_TextProperties() {
		return (EReference)customizedAttributeEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTextProperties() {
		return textPropertiesEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTextProperties_LabelText() {
		return (EAttribute)textPropertiesEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTextProperties_ReferencedLabelString() {
		return (EReference)textPropertiesEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTextProperties_TooltipText() {
		return (EAttribute)textPropertiesEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTextProperties_ReferencedTooltipString() {
		return (EReference)textPropertiesEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTextProperties_UnitLabelText() {
		return (EAttribute)textPropertiesEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTextProperties_ReferencedUnitLabelString() {
		return (EReference)textPropertiesEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIField() {
		return iFieldEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIField_Object() {
		return (EReference)iFieldEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIField_Feature() {
		return (EReference)iFieldEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldFeature() {
		return fieldFeatureEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldFeature_Feature() {
		return (EReference)fieldFeatureEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldFeature_SubFeature() {
		return (EReference)fieldFeatureEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldReference() {
		return fieldReferenceEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizeableField() {
		return customizeableFieldEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizeableField_ControlType() {
		return (EAttribute)customizeableFieldEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Parts() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Constraints() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Format() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_TextProperties() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldPartSpecification() {
		return fieldPartSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getFieldPartSpecification_Part() {
		return (EAttribute)fieldPartSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldPartSpecification_LayoutProperties() {
		return (EReference)fieldPartSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCommand() {
		return commandEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_Doc() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_ClientOnly() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_Name() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventFunction() {
		return eventFunctionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventFunction_Command() {
		return (EReference)eventFunctionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getEventFunction_PredefinedCommand() {
		return (EAttribute)eventFunctionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventMappingList() {
		return eventMappingListEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventMappingList_Mapping() {
		return (EReference)eventMappingListEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventMapping() {
		return eventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventMapping_EventFunctions() {
		return (EReference)eventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getControlEventMapping() {
		return controlEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getControlEventMapping_Event() {
		return (EAttribute)controlEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXMAWidgetEventMapping() {
		return xmaWidgetEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetEventMapping_Control() {
		return (EReference)xmaWidgetEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElementEventMapping() {
		return guiElementEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getGuiElementEventMapping_Control() {
		return (EReference)guiElementEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getInitEventMapping() {
		return initEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEnterEventMapping() {
		return enterEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLeaveEventMapping() {
		return leaveEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataMappingList() {
		return dataMappingListEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMappingList_Mapping() {
		return (EReference)dataMappingListEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataMapping() {
		return dataMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_Control() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_BindingElement() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_Field() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageDefinition() {
		return pageDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPage() {
		return pageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_ComposeLayout() {
		return (EReference)pageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Dataobjects() {
		return (EReference)pageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_DataMapping() {
		return (EReference)pageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Commands() {
		return (EReference)pageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Events() {
		return (EReference)pageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_ConditionsBlock() {
		return (EReference)pageEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Definitions() {
		return (EReference)pageEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmadslPage() {
		return xmadslPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslPage_Abstract() {
		return (EAttribute)xmadslPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslPage_Name() {
		return (EAttribute)xmadslPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Template() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Dependencies() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_PageProperties() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Extensions() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_LabelText() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_PageProperty() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageCustomization() {
		return pageCustomizationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPageCustomization_PageToCustomize() {
		return (EReference)pageCustomizationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPageCustomization_Composites() {
		return (EReference)pageCustomizationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElement() {
		return guiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposite() {
		return compositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposite_Content() {
		return (EReference)compositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposedElement() {
		return composedElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComplexElement() {
		return complexElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSimpleElement() {
		return simpleElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIDummy() {
		return iDummyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIDummy_Name() {
		return (EAttribute)iDummyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIGuiElementWithEvent() {
		return iGuiElementWithEventEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElementWithEvent_dummy() {
		return guiElementWithEvent_dummyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getGuiElementWithEvent_dummy_DummyProperty() {
		return (EAttribute)guiElementWithEvent_dummyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getText() {
		return textEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getText_Alignment() {
		return (EAttribute)textEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_Constraints() {
		return (EReference)textEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_Format() {
		return (EReference)textEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_TextProperties() {
		return (EReference)textEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCombo() {
		return comboEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_Constraints() {
		return (EReference)comboEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_Format() {
		return (EReference)comboEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_TextProperties() {
		return (EReference)comboEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCheckBox() {
		return checkBoxEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_LabelText() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_Constraints() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_Format() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_TextProperties() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getListBox() {
		return listBoxEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getListBox_TextProperties() {
		return (EReference)listBoxEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTree() {
		return treeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTree_TextProperties() {
		return (EReference)treeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getRadioButton() {
		return radioButtonEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_LabelText() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_Constraints() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_Format() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_TextProperties() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSeperator() {
		return seperatorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSeperator_Orientation() {
		return (EAttribute)seperatorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLabel() {
		return labelEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabel_LabelText() {
		return (EReference)labelEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabel_ImageUri() {
		return (EAttribute)labelEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabel_ImageUriReference() {
		return (EReference)labelEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabel_Alignment() {
		return (EAttribute)labelEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getImage() {
		return imageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImage_ImageUri() {
		return (EAttribute)imageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getImage_ImageUriReference() {
		return (EReference)imageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getButton() {
		return buttonEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getButton_LabelText() {
		return (EReference)buttonEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getButton_ImageUri() {
		return (EAttribute)buttonEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getButton_ImageUriReference() {
		return (EReference)buttonEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getButton_Alignment() {
		return (EAttribute)buttonEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIComposite() {
		return iCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmadslComposite() {
		return xmadslCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslComposite_Scrollable() {
		return (EReference)xmadslCompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslComposite_LabelText() {
		return (EReference)xmadslCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslComposite_VerticalPartitioning() {
		return (EAttribute)xmadslCompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslComposite_HorizontalPartitioning() {
		return (EAttribute)xmadslCompositeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedComposite() {
		return referencedCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedComposite_Reference() {
		return (EReference)referencedCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTable() {
		return tableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Object() {
		return (EReference)tableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Key() {
		return (EReference)tableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTable_ColumnMinWidth() {
		return (EAttribute)tableEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_MultiSelection() {
		return (EReference)tableEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_CheckSelection() {
		return (EReference)tableEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Columns() {
		return (EReference)tableEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTableColumn() {
		return tableColumnEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Object() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_AttributeHolder() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Visible() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_CustomLabelText() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_Align() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_Width() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_RelativeWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_MinWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_MaxWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(8);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getITabFolder() {
		return iTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getITabFolder_TabItems() {
		return (EReference)iTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabFolder() {
		return tabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedTabFolder() {
		return referencedTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedTabFolder_Reference() {
		return (EReference)referencedTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getITabPage() {
		return iTabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabPage() {
		return tabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabPage_Name() {
		return (EAttribute)tabPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_LabelText() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_Scrollable() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_FieldFlags() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_PageProperties() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedTabPage() {
		return referencedTabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedTabPage_Reference() {
		return (EReference)referencedTabPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLayoutDataProperty() {
		return layoutDataPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageProperty() {
		return pagePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getHeightProperty() {
		return heightPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getHeightProperty_Height() {
		return (EAttribute)heightPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getWidthProperty() {
		return widthPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getWidthProperty_Width() {
		return (EAttribute)widthPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentProperty() {
		return attachmentPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAttachmentProperty_OwnPosition() {
		return (EAttribute)attachmentPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getAttachmentProperty_AttachmentSpecification() {
		return (EReference)attachmentPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleProperty() {
		return stylePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleProperty_Styles() {
		return (EReference)stylePropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTitleButtonsProperty() {
		return titleButtonsPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTitleButtonsProperty_Button() {
		return (EAttribute)titleButtonsPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getResizeableProperty() {
		return resizeablePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getResizeableProperty_Resizeable() {
		return (EAttribute)resizeablePropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAlignmentProperty() {
		return alignmentPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAlignmentProperty_HAlignment() {
		return (EAttribute)alignmentPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAlignmentProperty_VAlignment() {
		return (EAttribute)alignmentPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposeData() {
		return composeDataEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Tabulators() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_ReferencedTabulator() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_CellWidths() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getComposeData_CellWidthsCount() {
		return (EAttribute)composeDataEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_CellHeights() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Spacing() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Margin() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPaddingWidth() {
		return paddingWidthEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPaddingWidth_Width() {
		return (EAttribute)paddingWidthEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIAttachmentPosition() {
		return iAttachmentPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Name() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Width() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Relative() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_Offset() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_ReferencedPosition() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_AdditionalOffset() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorDefinition() {
		return tabulatorDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorDefinition_Name() {
		return (EAttribute)tabulatorDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorDefinition_Tabulators() {
		return (EReference)tabulatorDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStringDefinition() {
		return stringDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStringDefinition_Name() {
		return (EAttribute)stringDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStringDefinition_Text() {
		return (EAttribute)stringDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIntegerDefinition() {
		return integerDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIntegerDefinition_Name() {
		return (EAttribute)integerDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIntegerDefinition_Value() {
		return (EAttribute)integerDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getImageUri() {
		return imageUriEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImageUri_Name() {
		return (EAttribute)imageUriEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImageUri_Source() {
		return (EAttribute)imageUriEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDefinitions() {
		return definitionsEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Strings() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Integers() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_ImageSources() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Positions() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Tabulators() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorPosition() {
		return tabulatorPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_FloatingPosition() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_FillingPosition() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_DeltaSpecification() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentPosition() {
		return attachmentPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getContent() {
		return contentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_WidgetSetItems() {
		return (EReference)contentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_ContainerItems() {
		return (EReference)contentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_Elements() {
		return (EReference)contentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_ComposeLayout() {
		return (EReference)contentEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSetOfGuiElements() {
		return setOfGuiElementsEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSetOfGuiElements_Elements() {
		return (EReference)setOfGuiElementsEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentSpecification() {
		return attachmentSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getNoneAttachment() {
		return noneAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getNoneAttachment_None() {
		return (EAttribute)noneAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getParentAttachment() {
		return parentAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getParentAttachment_Distance() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getParentAttachment_Relative() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getParentAttachment_Offset() {
		return (EReference)parentAttachmentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIElementOnWhichCanBeAttached() {
		return iElementOnWhichCanBeAttachedEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSiblingAttachment() {
		return siblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSiblingAttachment_SiblingPosition() {
		return (EAttribute)siblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSiblingAttachment_Offset() {
		return (EReference)siblingAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAbsoluteSiblingAttachment() {
		return absoluteSiblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getAbsoluteSiblingAttachment_Sibling() {
		return (EReference)absoluteSiblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAbsoluteSiblingAttachment_FieldPart() {
		return (EAttribute)absoluteSiblingAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getRelativeSiblingAttachment() {
		return relativeSiblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getRelativeSiblingAttachment_RelativeSibling() {
		return (EAttribute)relativeSiblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorAttachment() {
		return tabulatorAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorAttachment_TabIndex() {
		return (EAttribute)tabulatorAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorAttachment_Sibling() {
		return (EReference)tabulatorAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorAttachment_Offset() {
		return (EReference)tabulatorAttachmentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getOffset() {
		return offsetEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getOffset_Negative() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getOffset_Value() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldVariable() {
		return fieldVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldVariable_Field() {
		return (EReference)fieldVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getFieldVariable_Access() {
		return (EAttribute)fieldVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmaVariable() {
		return xmaVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmaVariable_Reference() {
		return (EReference)xmaVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmaVariable_Access() {
		return (EAttribute)xmaVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizeComponentModel() {
		return customizeComponentModelEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Imports() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Component() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Customizations() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfGuiElement() {
		return customizationOfGuiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfGuiElement_Page() {
		return (EReference)customizationOfGuiElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfComposite() {
		return customizationOfCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_Composite() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_ComposeLayout() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_AddedElements() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizationOfComposite_VerticalPartitioning() {
		return (EAttribute)customizationOfCompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizationOfComposite_HorizontalPartitioning() {
		return (EAttribute)customizationOfCompositeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_AddedWidgetSetItems() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfTabFolder() {
		return customizationOfTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfTabFolder_TabFolder() {
		return (EReference)customizationOfTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfTabFolder_AddedTabItems() {
		return (EReference)customizationOfTabFolderEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIElementWithLayoutData() {
		return iElementWithLayoutDataEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIElementWithLayoutData_LayoutProperties() {
		return (EReference)iElementWithLayoutDataEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIElementWithLayoutData_FieldFlags() {
		return (EReference)iElementWithLayoutDataEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleSpecification() {
		return styleSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleSpecification_ModelElements() {
		return (EReference)styleSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleSpecification_StyleProperty() {
		return (EReference)styleSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getModelElementSpecification() {
		return modelElementSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getModelElementSpecification_ModelElement() {
		return (EReference)modelElementSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getModelElementSpecification_CombinedElements() {
		return (EReference)modelElementSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSelectedModelElement() {
		return selectedModelElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_Type() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_Name() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_FirstChild() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_MiddleChild() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_LastChild() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCombinedModelElement() {
		return combinedModelElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCombinedModelElement_Child() {
		return (EAttribute)combinedModelElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCombinedModelElement_Next() {
		return (EAttribute)combinedModelElementEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombinedModelElement_ModelElement() {
		return (EReference)combinedModelElementEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleSpecificationProperty() {
		return styleSpecificationPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyFieldPart() {
		return stylePropertyFieldPartEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFieldPart_Parts() {
		return (EReference)stylePropertyFieldPartEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyLeft() {
		return stylePropertyLeftEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyLeft_AttachmentSpecification() {
		return (EReference)stylePropertyLeftEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyRight() {
		return stylePropertyRightEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyRight_AttachmentSpecification() {
		return (EReference)stylePropertyRightEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTop() {
		return stylePropertyTopEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTop_AttachmentSpecification() {
		return (EReference)stylePropertyTopEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBottom() {
		return stylePropertyBottomEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyBottom_AttachmentSpecification() {
		return (EReference)stylePropertyBottomEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyWidth() {
		return stylePropertyWidthEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyWidth_Width() {
		return (EAttribute)stylePropertyWidthEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyHeight() {
		return stylePropertyHeightEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyHeight_Height() {
		return (EAttribute)stylePropertyHeightEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTabulator() {
		return stylePropertyTabulatorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTabulator_Tabulators() {
		return (EReference)stylePropertyTabulatorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBorder() {
		return stylePropertyBorderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBorder_NoneBorder() {
		return (EAttribute)stylePropertyBorderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyForegroundColor() {
		return stylePropertyForegroundColorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Red() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Blue() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Green() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_ColorID() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBackgroundColor() {
		return stylePropertyBackgroundColorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Red() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Blue() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Green() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_ColorID() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyWidgetVariant() {
		return stylePropertyWidgetVariantEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyWidgetVariant_Name() {
		return (EAttribute)stylePropertyWidgetVariantEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getEventType() {
		return eventTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getPredefinedCommand() {
		return predefinedCommandEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getAttachmentOwnPosition() {
		return attachmentOwnPositionEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getRelativeSibling() {
		return relativeSiblingEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getAttachmentSiblingPosition() {
		return attachmentSiblingPositionEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getContentAlignment() {
		return contentAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getHorizontalAlignment() {
		return horizontalAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getVerticalAlignment() {
		return verticalAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getOrientation() {
		return orientationEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getControlType() {
		return controlTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getFieldPart() {
		return fieldPartEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getTitleButton() {
		return titleButtonEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getStyleSelectorType() {
		return styleSelectorTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public PomFactory getPomFactory() {
		return (PomFactory)getEFactoryInstance();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private boolean isCreated = false;

	/**
	 * Creates the meta-model objects for the package.  This method is
	 * guarded to have no affect on any invocation but its first.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void createPackageContents() {
		if (isCreated) return;
		isCreated = true;

		// Create classes and their features
		dataBindingElementEClass = createEClass(DATA_BINDING_ELEMENT);

		fieldFlagEClass = createEClass(FIELD_FLAG);
		createEReference(fieldFlagEClass, FIELD_FLAG__EXPRESSION);

		enabledFlagEClass = createEClass(ENABLED_FLAG);

		mandatoryFlagEClass = createEClass(MANDATORY_FLAG);

		visibleFlagEClass = createEClass(VISIBLE_FLAG);

		collapsedFlagEClass = createEClass(COLLAPSED_FLAG);

		editableFlagEClass = createEClass(EDITABLE_FLAG);

		labelTextEClass = createEClass(LABEL_TEXT);
		createEAttribute(labelTextEClass, LABEL_TEXT__LABEL);
		createEReference(labelTextEClass, LABEL_TEXT__REFERENCED_LABEL_STRING);

		objectPropertyEClass = createEClass(OBJECT_PROPERTY);
		createEAttribute(objectPropertyEClass, OBJECT_PROPERTY__PUBLIC);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__TYPE);
		createEAttribute(objectPropertyEClass, OBJECT_PROPERTY__NAME);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__READ_ONLY);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__REQUIRED);

		componentEClass = createEClass(COMPONENT);
		createEReference(componentEClass, COMPONENT__XMA_COMPONENT);
		createEReference(componentEClass, COMPONENT__STYLE_PROPERTY);
		createEReference(componentEClass, COMPONENT__DEPENDENCIES);
		createEReference(componentEClass, COMPONENT__COMPONENT_PROPERTIES);
		createEReference(componentEClass, COMPONENT__DATAOBJECTS);
		createEReference(componentEClass, COMPONENT__CUSTOMIZED_ATTRIBUTE_LIST);
		createEReference(componentEClass, COMPONENT__COMMANDS);
		createEReference(componentEClass, COMPONENT__EVENTS);
		createEReference(componentEClass, COMPONENT__CONDITIONS_BLOCK);
		createEReference(componentEClass, COMPONENT__PAGES);
		createEReference(componentEClass, COMPONENT__DEFINITIONS);

		referencedXMAGuiElementEClass = createEClass(REFERENCED_XMA_GUI_ELEMENT);

		referencedXMAPageEClass = createEClass(REFERENCED_XMA_PAGE);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__XMA_PAGE);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__FIELD_FLAGS);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__DEPENDENCIES);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__GRAY_LOGIC);

		referencedXMACompositeEClass = createEClass(REFERENCED_XMA_COMPOSITE);
		createEReference(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__XMA_COMPOSITE);
		createEReference(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__FIELD_FLAGS);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__REFERENCING_PAGE_COMPOSITE);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__VERTICAL_PARTITIONING);

		grayLogicEClass = createEClass(GRAY_LOGIC);
		createEReference(grayLogicEClass, GRAY_LOGIC__WIDGET_GRAY_LOGIC);

		xmaWidgetGrayLogicEClass = createEClass(XMA_WIDGET_GRAY_LOGIC);
		createEReference(xmaWidgetGrayLogicEClass, XMA_WIDGET_GRAY_LOGIC__XMA_WIDGET);
		createEReference(xmaWidgetGrayLogicEClass, XMA_WIDGET_GRAY_LOGIC__FIELD_FLAGS);

		dataObjectVariableEClass = createEClass(DATA_OBJECT_VARIABLE);
		createEReference(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__TYPE);
		createEAttribute(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__IS_COLLECTION);
		createEAttribute(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__NAME);
		createEReference(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__CUSTOMIZED_ATTRIBUTES);

		customizeAttributeListEClass = createEClass(CUSTOMIZE_ATTRIBUTE_LIST);
		createEReference(customizeAttributeListEClass, CUSTOMIZE_ATTRIBUTE_LIST__CUSTOMIZED_ATTRIBUTES);

		customizedAttributeEClass = createEClass(CUSTOMIZED_ATTRIBUTE);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__FIELD);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__CONSTRAINTS);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__FORMAT);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__ATTRIBUT_FLAG);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__ATTRIBUT_FLAGS);
		createEReference(customizedAttributeEClass, CUSTOMIZED_ATTRIBUTE__TEXT_PROPERTIES);

		textPropertiesEClass = createEClass(TEXT_PROPERTIES);
		createEAttribute(textPropertiesEClass, TEXT_PROPERTIES__LABEL_TEXT);
		createEReference(textPropertiesEClass, TEXT_PROPERTIES__REFERENCED_LABEL_STRING);
		createEAttribute(textPropertiesEClass, TEXT_PROPERTIES__TOOLTIP_TEXT);
		createEReference(textPropertiesEClass, TEXT_PROPERTIES__REFERENCED_TOOLTIP_STRING);
		createEAttribute(textPropertiesEClass, TEXT_PROPERTIES__UNIT_LABEL_TEXT);
		createEReference(textPropertiesEClass, TEXT_PROPERTIES__REFERENCED_UNIT_LABEL_STRING);

		iFieldEClass = createEClass(IFIELD);
		createEReference(iFieldEClass, IFIELD__OBJECT);
		createEReference(iFieldEClass, IFIELD__FEATURE);

		fieldFeatureEClass = createEClass(FIELD_FEATURE);
		createEReference(fieldFeatureEClass, FIELD_FEATURE__FEATURE);
		createEReference(fieldFeatureEClass, FIELD_FEATURE__SUB_FEATURE);

		fieldReferenceEClass = createEClass(FIELD_REFERENCE);

		customizeableFieldEClass = createEClass(CUSTOMIZEABLE_FIELD);
		createEAttribute(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__CONTROL_TYPE);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__PARTS);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__CONSTRAINTS);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__FORMAT);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__TEXT_PROPERTIES);

		fieldPartSpecificationEClass = createEClass(FIELD_PART_SPECIFICATION);
		createEAttribute(fieldPartSpecificationEClass, FIELD_PART_SPECIFICATION__PART);
		createEReference(fieldPartSpecificationEClass, FIELD_PART_SPECIFICATION__LAYOUT_PROPERTIES);

		commandEClass = createEClass(COMMAND);
		createEAttribute(commandEClass, COMMAND__DOC);
		createEAttribute(commandEClass, COMMAND__CLIENT_ONLY);
		createEAttribute(commandEClass, COMMAND__NAME);

		eventFunctionEClass = createEClass(EVENT_FUNCTION);
		createEReference(eventFunctionEClass, EVENT_FUNCTION__COMMAND);
		createEAttribute(eventFunctionEClass, EVENT_FUNCTION__PREDEFINED_COMMAND);

		eventMappingListEClass = createEClass(EVENT_MAPPING_LIST);
		createEReference(eventMappingListEClass, EVENT_MAPPING_LIST__MAPPING);

		eventMappingEClass = createEClass(EVENT_MAPPING);
		createEReference(eventMappingEClass, EVENT_MAPPING__EVENT_FUNCTIONS);

		controlEventMappingEClass = createEClass(CONTROL_EVENT_MAPPING);
		createEAttribute(controlEventMappingEClass, CONTROL_EVENT_MAPPING__EVENT);

		xmaWidgetEventMappingEClass = createEClass(XMA_WIDGET_EVENT_MAPPING);
		createEReference(xmaWidgetEventMappingEClass, XMA_WIDGET_EVENT_MAPPING__CONTROL);

		guiElementEventMappingEClass = createEClass(GUI_ELEMENT_EVENT_MAPPING);
		createEReference(guiElementEventMappingEClass, GUI_ELEMENT_EVENT_MAPPING__CONTROL);

		initEventMappingEClass = createEClass(INIT_EVENT_MAPPING);

		enterEventMappingEClass = createEClass(ENTER_EVENT_MAPPING);

		leaveEventMappingEClass = createEClass(LEAVE_EVENT_MAPPING);

		dataMappingListEClass = createEClass(DATA_MAPPING_LIST);
		createEReference(dataMappingListEClass, DATA_MAPPING_LIST__MAPPING);

		dataMappingEClass = createEClass(DATA_MAPPING);
		createEReference(dataMappingEClass, DATA_MAPPING__CONTROL);
		createEReference(dataMappingEClass, DATA_MAPPING__BINDING_ELEMENT);
		createEReference(dataMappingEClass, DATA_MAPPING__FIELD);

		pageDefinitionEClass = createEClass(PAGE_DEFINITION);

		pageEClass = createEClass(PAGE);
		createEReference(pageEClass, PAGE__COMPOSE_LAYOUT);
		createEReference(pageEClass, PAGE__DATAOBJECTS);
		createEReference(pageEClass, PAGE__DATA_MAPPING);
		createEReference(pageEClass, PAGE__COMMANDS);
		createEReference(pageEClass, PAGE__EVENTS);
		createEReference(pageEClass, PAGE__CONDITIONS_BLOCK);
		createEReference(pageEClass, PAGE__DEFINITIONS);

		xmadslPageEClass = createEClass(XMADSL_PAGE);
		createEAttribute(xmadslPageEClass, XMADSL_PAGE__ABSTRACT);
		createEAttribute(xmadslPageEClass, XMADSL_PAGE__NAME);
		createEReference(xmadslPageEClass, XMADSL_PAGE__LABEL_TEXT);
		createEReference(xmadslPageEClass, XMADSL_PAGE__PAGE_PROPERTY);
		createEReference(xmadslPageEClass, XMADSL_PAGE__TEMPLATE);
		createEReference(xmadslPageEClass, XMADSL_PAGE__DEPENDENCIES);
		createEReference(xmadslPageEClass, XMADSL_PAGE__PAGE_PROPERTIES);
		createEReference(xmadslPageEClass, XMADSL_PAGE__EXTENSIONS);

		pageCustomizationEClass = createEClass(PAGE_CUSTOMIZATION);
		createEReference(pageCustomizationEClass, PAGE_CUSTOMIZATION__PAGE_TO_CUSTOMIZE);
		createEReference(pageCustomizationEClass, PAGE_CUSTOMIZATION__COMPOSITES);

		guiElementEClass = createEClass(GUI_ELEMENT);

		compositeEClass = createEClass(COMPOSITE);
		createEReference(compositeEClass, COMPOSITE__CONTENT);

		composedElementEClass = createEClass(COMPOSED_ELEMENT);

		complexElementEClass = createEClass(COMPLEX_ELEMENT);

		simpleElementEClass = createEClass(SIMPLE_ELEMENT);

		iDummyEClass = createEClass(IDUMMY);
		createEAttribute(iDummyEClass, IDUMMY__NAME);

		iGuiElementWithEventEClass = createEClass(IGUI_ELEMENT_WITH_EVENT);

		guiElementWithEvent_dummyEClass = createEClass(GUI_ELEMENT_WITH_EVENT_DUMMY);
		createEAttribute(guiElementWithEvent_dummyEClass, GUI_ELEMENT_WITH_EVENT_DUMMY__DUMMY_PROPERTY);

		textEClass = createEClass(TEXT);
		createEAttribute(textEClass, TEXT__ALIGNMENT);
		createEReference(textEClass, TEXT__CONSTRAINTS);
		createEReference(textEClass, TEXT__FORMAT);
		createEReference(textEClass, TEXT__TEXT_PROPERTIES);

		comboEClass = createEClass(COMBO);
		createEReference(comboEClass, COMBO__CONSTRAINTS);
		createEReference(comboEClass, COMBO__FORMAT);
		createEReference(comboEClass, COMBO__TEXT_PROPERTIES);

		checkBoxEClass = createEClass(CHECK_BOX);
		createEReference(checkBoxEClass, CHECK_BOX__LABEL_TEXT);
		createEReference(checkBoxEClass, CHECK_BOX__CONSTRAINTS);
		createEReference(checkBoxEClass, CHECK_BOX__FORMAT);
		createEReference(checkBoxEClass, CHECK_BOX__TEXT_PROPERTIES);

		listBoxEClass = createEClass(LIST_BOX);
		createEReference(listBoxEClass, LIST_BOX__TEXT_PROPERTIES);

		treeEClass = createEClass(TREE);
		createEReference(treeEClass, TREE__TEXT_PROPERTIES);

		radioButtonEClass = createEClass(RADIO_BUTTON);
		createEReference(radioButtonEClass, RADIO_BUTTON__LABEL_TEXT);
		createEReference(radioButtonEClass, RADIO_BUTTON__CONSTRAINTS);
		createEReference(radioButtonEClass, RADIO_BUTTON__FORMAT);
		createEReference(radioButtonEClass, RADIO_BUTTON__TEXT_PROPERTIES);

		seperatorEClass = createEClass(SEPERATOR);
		createEAttribute(seperatorEClass, SEPERATOR__ORIENTATION);

		labelEClass = createEClass(LABEL);
		createEReference(labelEClass, LABEL__LABEL_TEXT);
		createEAttribute(labelEClass, LABEL__IMAGE_URI);
		createEReference(labelEClass, LABEL__IMAGE_URI_REFERENCE);
		createEAttribute(labelEClass, LABEL__ALIGNMENT);

		imageEClass = createEClass(IMAGE);
		createEAttribute(imageEClass, IMAGE__IMAGE_URI);
		createEReference(imageEClass, IMAGE__IMAGE_URI_REFERENCE);

		buttonEClass = createEClass(BUTTON);
		createEReference(buttonEClass, BUTTON__LABEL_TEXT);
		createEAttribute(buttonEClass, BUTTON__IMAGE_URI);
		createEReference(buttonEClass, BUTTON__IMAGE_URI_REFERENCE);
		createEAttribute(buttonEClass, BUTTON__ALIGNMENT);

		iCompositeEClass = createEClass(ICOMPOSITE);

		xmadslCompositeEClass = createEClass(XMADSL_COMPOSITE);
		createEReference(xmadslCompositeEClass, XMADSL_COMPOSITE__LABEL_TEXT);
		createEReference(xmadslCompositeEClass, XMADSL_COMPOSITE__SCROLLABLE);
		createEAttribute(xmadslCompositeEClass, XMADSL_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEAttribute(xmadslCompositeEClass, XMADSL_COMPOSITE__VERTICAL_PARTITIONING);

		referencedCompositeEClass = createEClass(REFERENCED_COMPOSITE);
		createEReference(referencedCompositeEClass, REFERENCED_COMPOSITE__REFERENCE);

		tableEClass = createEClass(TABLE);
		createEReference(tableEClass, TABLE__OBJECT);
		createEReference(tableEClass, TABLE__KEY);
		createEAttribute(tableEClass, TABLE__COLUMN_MIN_WIDTH);
		createEReference(tableEClass, TABLE__MULTI_SELECTION);
		createEReference(tableEClass, TABLE__CHECK_SELECTION);
		createEReference(tableEClass, TABLE__COLUMNS);

		tableColumnEClass = createEClass(TABLE_COLUMN);
		createEReference(tableColumnEClass, TABLE_COLUMN__OBJECT);
		createEReference(tableColumnEClass, TABLE_COLUMN__ATTRIBUTE_HOLDER);
		createEReference(tableColumnEClass, TABLE_COLUMN__VISIBLE);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__CUSTOM_LABEL_TEXT);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__ALIGN);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__RELATIVE_WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__MIN_WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__MAX_WIDTH);

		iTabFolderEClass = createEClass(ITAB_FOLDER);
		createEReference(iTabFolderEClass, ITAB_FOLDER__TAB_ITEMS);

		tabFolderEClass = createEClass(TAB_FOLDER);

		referencedTabFolderEClass = createEClass(REFERENCED_TAB_FOLDER);
		createEReference(referencedTabFolderEClass, REFERENCED_TAB_FOLDER__REFERENCE);

		iTabPageEClass = createEClass(ITAB_PAGE);

		tabPageEClass = createEClass(TAB_PAGE);
		createEAttribute(tabPageEClass, TAB_PAGE__NAME);
		createEReference(tabPageEClass, TAB_PAGE__LABEL_TEXT);
		createEReference(tabPageEClass, TAB_PAGE__SCROLLABLE);
		createEReference(tabPageEClass, TAB_PAGE__FIELD_FLAGS);
		createEReference(tabPageEClass, TAB_PAGE__PAGE_PROPERTIES);

		referencedTabPageEClass = createEClass(REFERENCED_TAB_PAGE);
		createEReference(referencedTabPageEClass, REFERENCED_TAB_PAGE__REFERENCE);

		layoutDataPropertyEClass = createEClass(LAYOUT_DATA_PROPERTY);

		pagePropertyEClass = createEClass(PAGE_PROPERTY);

		heightPropertyEClass = createEClass(HEIGHT_PROPERTY);
		createEAttribute(heightPropertyEClass, HEIGHT_PROPERTY__HEIGHT);

		widthPropertyEClass = createEClass(WIDTH_PROPERTY);
		createEAttribute(widthPropertyEClass, WIDTH_PROPERTY__WIDTH);

		stylePropertyEClass = createEClass(STYLE_PROPERTY);
		createEReference(stylePropertyEClass, STYLE_PROPERTY__STYLES);

		titleButtonsPropertyEClass = createEClass(TITLE_BUTTONS_PROPERTY);
		createEAttribute(titleButtonsPropertyEClass, TITLE_BUTTONS_PROPERTY__BUTTON);

		resizeablePropertyEClass = createEClass(RESIZEABLE_PROPERTY);
		createEAttribute(resizeablePropertyEClass, RESIZEABLE_PROPERTY__RESIZEABLE);

		alignmentPropertyEClass = createEClass(ALIGNMENT_PROPERTY);
		createEAttribute(alignmentPropertyEClass, ALIGNMENT_PROPERTY__HALIGNMENT);
		createEAttribute(alignmentPropertyEClass, ALIGNMENT_PROPERTY__VALIGNMENT);

		composeDataEClass = createEClass(COMPOSE_DATA);
		createEReference(composeDataEClass, COMPOSE_DATA__TABULATORS);
		createEReference(composeDataEClass, COMPOSE_DATA__REFERENCED_TABULATOR);
		createEReference(composeDataEClass, COMPOSE_DATA__CELL_WIDTHS);
		createEAttribute(composeDataEClass, COMPOSE_DATA__CELL_WIDTHS_COUNT);
		createEReference(composeDataEClass, COMPOSE_DATA__CELL_HEIGHTS);
		createEReference(composeDataEClass, COMPOSE_DATA__SPACING);
		createEReference(composeDataEClass, COMPOSE_DATA__MARGIN);

		paddingWidthEClass = createEClass(PADDING_WIDTH);
		createEAttribute(paddingWidthEClass, PADDING_WIDTH__WIDTH);

		iAttachmentPositionEClass = createEClass(IATTACHMENT_POSITION);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__NAME);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__WIDTH);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__RELATIVE);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__OFFSET);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__REFERENCED_POSITION);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__ADDITIONAL_OFFSET);

		tabulatorDefinitionEClass = createEClass(TABULATOR_DEFINITION);
		createEAttribute(tabulatorDefinitionEClass, TABULATOR_DEFINITION__NAME);
		createEReference(tabulatorDefinitionEClass, TABULATOR_DEFINITION__TABULATORS);

		stringDefinitionEClass = createEClass(STRING_DEFINITION);
		createEAttribute(stringDefinitionEClass, STRING_DEFINITION__NAME);
		createEAttribute(stringDefinitionEClass, STRING_DEFINITION__TEXT);

		integerDefinitionEClass = createEClass(INTEGER_DEFINITION);
		createEAttribute(integerDefinitionEClass, INTEGER_DEFINITION__NAME);
		createEAttribute(integerDefinitionEClass, INTEGER_DEFINITION__VALUE);

		imageUriEClass = createEClass(IMAGE_URI);
		createEAttribute(imageUriEClass, IMAGE_URI__NAME);
		createEAttribute(imageUriEClass, IMAGE_URI__SOURCE);

		definitionsEClass = createEClass(DEFINITIONS);
		createEReference(definitionsEClass, DEFINITIONS__STRINGS);
		createEReference(definitionsEClass, DEFINITIONS__INTEGERS);
		createEReference(definitionsEClass, DEFINITIONS__IMAGE_SOURCES);
		createEReference(definitionsEClass, DEFINITIONS__POSITIONS);
		createEReference(definitionsEClass, DEFINITIONS__TABULATORS);

		tabulatorPositionEClass = createEClass(TABULATOR_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__FLOATING_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__FILLING_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__DELTA_SPECIFICATION);

		attachmentPositionEClass = createEClass(ATTACHMENT_POSITION);

		contentEClass = createEClass(CONTENT);
		createEReference(contentEClass, CONTENT__WIDGET_SET_ITEMS);
		createEReference(contentEClass, CONTENT__CONTAINER_ITEMS);
		createEReference(contentEClass, CONTENT__ELEMENTS);
		createEReference(contentEClass, CONTENT__COMPOSE_LAYOUT);

		setOfGuiElementsEClass = createEClass(SET_OF_GUI_ELEMENTS);
		createEReference(setOfGuiElementsEClass, SET_OF_GUI_ELEMENTS__ELEMENTS);

		attachmentPropertyEClass = createEClass(ATTACHMENT_PROPERTY);
		createEAttribute(attachmentPropertyEClass, ATTACHMENT_PROPERTY__OWN_POSITION);
		createEReference(attachmentPropertyEClass, ATTACHMENT_PROPERTY__ATTACHMENT_SPECIFICATION);

		attachmentSpecificationEClass = createEClass(ATTACHMENT_SPECIFICATION);

		noneAttachmentEClass = createEClass(NONE_ATTACHMENT);
		createEAttribute(noneAttachmentEClass, NONE_ATTACHMENT__NONE);

		parentAttachmentEClass = createEClass(PARENT_ATTACHMENT);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__DISTANCE);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__RELATIVE);
		createEReference(parentAttachmentEClass, PARENT_ATTACHMENT__OFFSET);

		iElementOnWhichCanBeAttachedEClass = createEClass(IELEMENT_ON_WHICH_CAN_BE_ATTACHED);

		siblingAttachmentEClass = createEClass(SIBLING_ATTACHMENT);
		createEAttribute(siblingAttachmentEClass, SIBLING_ATTACHMENT__SIBLING_POSITION);
		createEReference(siblingAttachmentEClass, SIBLING_ATTACHMENT__OFFSET);

		absoluteSiblingAttachmentEClass = createEClass(ABSOLUTE_SIBLING_ATTACHMENT);
		createEReference(absoluteSiblingAttachmentEClass, ABSOLUTE_SIBLING_ATTACHMENT__SIBLING);
		createEAttribute(absoluteSiblingAttachmentEClass, ABSOLUTE_SIBLING_ATTACHMENT__FIELD_PART);

		relativeSiblingAttachmentEClass = createEClass(RELATIVE_SIBLING_ATTACHMENT);
		createEAttribute(relativeSiblingAttachmentEClass, RELATIVE_SIBLING_ATTACHMENT__RELATIVE_SIBLING);

		tabulatorAttachmentEClass = createEClass(TABULATOR_ATTACHMENT);
		createEAttribute(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__TAB_INDEX);
		createEReference(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__SIBLING);
		createEReference(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__OFFSET);

		offsetEClass = createEClass(OFFSET);
		createEAttribute(offsetEClass, OFFSET__NEGATIVE);
		createEAttribute(offsetEClass, OFFSET__VALUE);

		fieldVariableEClass = createEClass(FIELD_VARIABLE);
		createEReference(fieldVariableEClass, FIELD_VARIABLE__FIELD);
		createEAttribute(fieldVariableEClass, FIELD_VARIABLE__ACCESS);

		xmaVariableEClass = createEClass(XMA_VARIABLE);
		createEReference(xmaVariableEClass, XMA_VARIABLE__REFERENCE);
		createEAttribute(xmaVariableEClass, XMA_VARIABLE__ACCESS);

		customizeComponentModelEClass = createEClass(CUSTOMIZE_COMPONENT_MODEL);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__IMPORTS);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__COMPONENT);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__CUSTOMIZATIONS);

		customizationOfGuiElementEClass = createEClass(CUSTOMIZATION_OF_GUI_ELEMENT);
		createEReference(customizationOfGuiElementEClass, CUSTOMIZATION_OF_GUI_ELEMENT__PAGE);

		customizationOfCompositeEClass = createEClass(CUSTOMIZATION_OF_COMPOSITE);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__COMPOSITE);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__COMPOSE_LAYOUT);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__ADDED_ELEMENTS);
		createEAttribute(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__VERTICAL_PARTITIONING);
		createEAttribute(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__ADDED_WIDGET_SET_ITEMS);

		customizationOfTabFolderEClass = createEClass(CUSTOMIZATION_OF_TAB_FOLDER);
		createEReference(customizationOfTabFolderEClass, CUSTOMIZATION_OF_TAB_FOLDER__TAB_FOLDER);
		createEReference(customizationOfTabFolderEClass, CUSTOMIZATION_OF_TAB_FOLDER__ADDED_TAB_ITEMS);

		iElementWithLayoutDataEClass = createEClass(IELEMENT_WITH_LAYOUT_DATA);
		createEReference(iElementWithLayoutDataEClass, IELEMENT_WITH_LAYOUT_DATA__LAYOUT_PROPERTIES);
		createEReference(iElementWithLayoutDataEClass, IELEMENT_WITH_LAYOUT_DATA__FIELD_FLAGS);

		styleSpecificationEClass = createEClass(STYLE_SPECIFICATION);
		createEReference(styleSpecificationEClass, STYLE_SPECIFICATION__MODEL_ELEMENTS);
		createEReference(styleSpecificationEClass, STYLE_SPECIFICATION__STYLE_PROPERTY);

		modelElementSpecificationEClass = createEClass(MODEL_ELEMENT_SPECIFICATION);
		createEReference(modelElementSpecificationEClass, MODEL_ELEMENT_SPECIFICATION__MODEL_ELEMENT);
		createEReference(modelElementSpecificationEClass, MODEL_ELEMENT_SPECIFICATION__COMBINED_ELEMENTS);

		selectedModelElementEClass = createEClass(SELECTED_MODEL_ELEMENT);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__TYPE);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__NAME);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__FIRST_CHILD);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__MIDDLE_CHILD);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__LAST_CHILD);

		combinedModelElementEClass = createEClass(COMBINED_MODEL_ELEMENT);
		createEAttribute(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__CHILD);
		createEAttribute(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__NEXT);
		createEReference(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__MODEL_ELEMENT);

		styleSpecificationPropertyEClass = createEClass(STYLE_SPECIFICATION_PROPERTY);

		stylePropertyFieldPartEClass = createEClass(STYLE_PROPERTY_FIELD_PART);
		createEReference(stylePropertyFieldPartEClass, STYLE_PROPERTY_FIELD_PART__PARTS);

		stylePropertyLeftEClass = createEClass(STYLE_PROPERTY_LEFT);
		createEReference(stylePropertyLeftEClass, STYLE_PROPERTY_LEFT__ATTACHMENT_SPECIFICATION);

		stylePropertyRightEClass = createEClass(STYLE_PROPERTY_RIGHT);
		createEReference(stylePropertyRightEClass, STYLE_PROPERTY_RIGHT__ATTACHMENT_SPECIFICATION);

		stylePropertyTopEClass = createEClass(STYLE_PROPERTY_TOP);
		createEReference(stylePropertyTopEClass, STYLE_PROPERTY_TOP__ATTACHMENT_SPECIFICATION);

		stylePropertyBottomEClass = createEClass(STYLE_PROPERTY_BOTTOM);
		createEReference(stylePropertyBottomEClass, STYLE_PROPERTY_BOTTOM__ATTACHMENT_SPECIFICATION);

		stylePropertyWidthEClass = createEClass(STYLE_PROPERTY_WIDTH);
		createEAttribute(stylePropertyWidthEClass, STYLE_PROPERTY_WIDTH__WIDTH);

		stylePropertyHeightEClass = createEClass(STYLE_PROPERTY_HEIGHT);
		createEAttribute(stylePropertyHeightEClass, STYLE_PROPERTY_HEIGHT__HEIGHT);

		stylePropertyTabulatorEClass = createEClass(STYLE_PROPERTY_TABULATOR);
		createEReference(stylePropertyTabulatorEClass, STYLE_PROPERTY_TABULATOR__TABULATORS);

		stylePropertyBorderEClass = createEClass(STYLE_PROPERTY_BORDER);
		createEAttribute(stylePropertyBorderEClass, STYLE_PROPERTY_BORDER__NONE_BORDER);

		stylePropertyForegroundColorEClass = createEClass(STYLE_PROPERTY_FOREGROUND_COLOR);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__RED);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__BLUE);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__GREEN);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__COLOR_ID);

		stylePropertyBackgroundColorEClass = createEClass(STYLE_PROPERTY_BACKGROUND_COLOR);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__RED);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__BLUE);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__GREEN);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__COLOR_ID);

		stylePropertyWidgetVariantEClass = createEClass(STYLE_PROPERTY_WIDGET_VARIANT);
		createEAttribute(stylePropertyWidgetVariantEClass, STYLE_PROPERTY_WIDGET_VARIANT__NAME);

		// Create enums
		eventTypeEEnum = createEEnum(EVENT_TYPE);
		predefinedCommandEEnum = createEEnum(PREDEFINED_COMMAND);
		attachmentOwnPositionEEnum = createEEnum(ATTACHMENT_OWN_POSITION);
		relativeSiblingEEnum = createEEnum(RELATIVE_SIBLING);
		attachmentSiblingPositionEEnum = createEEnum(ATTACHMENT_SIBLING_POSITION);
		contentAlignmentEEnum = createEEnum(CONTENT_ALIGNMENT);
		horizontalAlignmentEEnum = createEEnum(HORIZONTAL_ALIGNMENT);
		verticalAlignmentEEnum = createEEnum(VERTICAL_ALIGNMENT);
		orientationEEnum = createEEnum(ORIENTATION);
		controlTypeEEnum = createEEnum(CONTROL_TYPE);
		fieldPartEEnum = createEEnum(FIELD_PART);
		titleButtonEEnum = createEEnum(TITLE_BUTTON);
		styleSelectorTypeEEnum = createEEnum(STYLE_SELECTOR_TYPE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private boolean isInitialized = false;

	/**
	 * Complete the initialization of the package and its meta-model.  This
	 * method is guarded to have no affect on any invocation but its first.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void initializePackageContents() {
		if (isInitialized) return;
		isInitialized = true;

		// Initialize package
		setName(eNAME);
		setNsPrefix(eNS_PREFIX);
		setNsURI(eNS_URI);

		// Obtain other dependent packages
		CorePackage theCorePackage = (CorePackage)EPackage.Registry.INSTANCE.getEPackage(CorePackage.eNS_URI);
		GuidesignPackage theGuidesignPackage = (GuidesignPackage)EPackage.Registry.INSTANCE.getEPackage(GuidesignPackage.eNS_URI);
		DomPackage theDomPackage = (DomPackage)EPackage.Registry.INSTANCE.getEPackage(DomPackage.eNS_URI);
		FlexPackage theFlexPackage = (FlexPackage)EPackage.Registry.INSTANCE.getEPackage(FlexPackage.eNS_URI);

		// Create type parameters

		// Set bounds for type parameters

		// Add supertypes to classes
		enabledFlagEClass.getESuperTypes().add(this.getFieldFlag());
		mandatoryFlagEClass.getESuperTypes().add(this.getFieldFlag());
		visibleFlagEClass.getESuperTypes().add(this.getFieldFlag());
		collapsedFlagEClass.getESuperTypes().add(this.getFieldFlag());
		editableFlagEClass.getESuperTypes().add(this.getFieldFlag());
		objectPropertyEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		objectPropertyEClass.getESuperTypes().add(this.getDataBindingElement());
		componentEClass.getESuperTypes().add(theCorePackage.getModelElement());
		componentEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		referencedXMAPageEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		referencedXMAPageEClass.getESuperTypes().add(this.getPage());
		referencedXMACompositeEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		referencedXMACompositeEClass.getESuperTypes().add(this.getIComposite());
		fieldReferenceEClass.getESuperTypes().add(this.getIField());
		customizeableFieldEClass.getESuperTypes().add(this.getIField());
		customizeableFieldEClass.getESuperTypes().add(this.getSimpleElement());
		customizeableFieldEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		controlEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		xmaWidgetEventMappingEClass.getESuperTypes().add(this.getControlEventMapping());
		guiElementEventMappingEClass.getESuperTypes().add(this.getControlEventMapping());
		initEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		enterEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		leaveEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		pageEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		pageEClass.getESuperTypes().add(this.getPageDefinition());
		pageEClass.getESuperTypes().add(this.getComposite());
		xmadslPageEClass.getESuperTypes().add(this.getPage());
		pageCustomizationEClass.getESuperTypes().add(this.getPageDefinition());
		compositeEClass.getESuperTypes().add(this.getGuiElement());
		composedElementEClass.getESuperTypes().add(this.getGuiElement());
		composedElementEClass.getESuperTypes().add(this.getIElementOnWhichCanBeAttached());
		complexElementEClass.getESuperTypes().add(this.getComposedElement());
		simpleElementEClass.getESuperTypes().add(this.getComposedElement());
		simpleElementEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		iGuiElementWithEventEClass.getESuperTypes().add(this.getIDummy());
		guiElementWithEvent_dummyEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		textEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		textEClass.getESuperTypes().add(this.getDataBindingElement());
		textEClass.getESuperTypes().add(this.getSimpleElement());
		textEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		comboEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		comboEClass.getESuperTypes().add(this.getDataBindingElement());
		comboEClass.getESuperTypes().add(this.getSimpleElement());
		comboEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		checkBoxEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		checkBoxEClass.getESuperTypes().add(this.getDataBindingElement());
		checkBoxEClass.getESuperTypes().add(this.getSimpleElement());
		checkBoxEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		listBoxEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		listBoxEClass.getESuperTypes().add(this.getSimpleElement());
		listBoxEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		treeEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		treeEClass.getESuperTypes().add(this.getSimpleElement());
		treeEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		radioButtonEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		radioButtonEClass.getESuperTypes().add(this.getDataBindingElement());
		radioButtonEClass.getESuperTypes().add(this.getSimpleElement());
		radioButtonEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		seperatorEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		seperatorEClass.getESuperTypes().add(this.getSimpleElement());
		seperatorEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		labelEClass.getESuperTypes().add(this.getSimpleElement());
		imageEClass.getESuperTypes().add(this.getSimpleElement());
		imageEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		buttonEClass.getESuperTypes().add(this.getSimpleElement());
		buttonEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		iCompositeEClass.getESuperTypes().add(this.getComposite());
		iCompositeEClass.getESuperTypes().add(this.getComplexElement());
		xmadslCompositeEClass.getESuperTypes().add(this.getIComposite());
		xmadslCompositeEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		referencedCompositeEClass.getESuperTypes().add(this.getIComposite());
		tableEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		tableEClass.getESuperTypes().add(this.getComplexElement());
		tableEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		tableEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		iTabFolderEClass.getESuperTypes().add(this.getComplexElement());
		tabFolderEClass.getESuperTypes().add(this.getITabFolder());
		tabFolderEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		referencedTabFolderEClass.getESuperTypes().add(this.getITabFolder());
		iTabPageEClass.getESuperTypes().add(this.getPage());
		tabPageEClass.getESuperTypes().add(this.getITabPage());
		referencedTabPageEClass.getESuperTypes().add(this.getITabPage());
		heightPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		heightPropertyEClass.getESuperTypes().add(this.getPageProperty());
		widthPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		widthPropertyEClass.getESuperTypes().add(this.getPageProperty());
		stylePropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		stylePropertyEClass.getESuperTypes().add(this.getPageProperty());
		titleButtonsPropertyEClass.getESuperTypes().add(this.getPageProperty());
		resizeablePropertyEClass.getESuperTypes().add(this.getPageProperty());
		alignmentPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		iAttachmentPositionEClass.getESuperTypes().add(this.getIElementOnWhichCanBeAttached());
		tabulatorPositionEClass.getESuperTypes().add(this.getIAttachmentPosition());
		attachmentPositionEClass.getESuperTypes().add(this.getIAttachmentPosition());
		attachmentPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		noneAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		parentAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		siblingAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		absoluteSiblingAttachmentEClass.getESuperTypes().add(this.getSiblingAttachment());
		relativeSiblingAttachmentEClass.getESuperTypes().add(this.getSiblingAttachment());
		tabulatorAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		fieldVariableEClass.getESuperTypes().add(theCorePackage.getVariable());
		xmaVariableEClass.getESuperTypes().add(theCorePackage.getVariable());
		customizationOfCompositeEClass.getESuperTypes().add(this.getCustomizationOfGuiElement());
		customizationOfTabFolderEClass.getESuperTypes().add(this.getCustomizationOfGuiElement());
		iElementWithLayoutDataEClass.getESuperTypes().add(this.getIDummy());
		styleSpecificationEClass.getESuperTypes().add(theCorePackage.getStyle());
		stylePropertyFieldPartEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyLeftEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyRightEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTopEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBottomEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyWidthEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyHeightEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTabulatorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBorderEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyForegroundColorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBackgroundColorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyWidgetVariantEClass.getESuperTypes().add(this.getStyleSpecificationProperty());

		// Initialize classes and features; add operations and parameters
		initEClass(dataBindingElementEClass, DataBindingElement.class, "DataBindingElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(fieldFlagEClass, FieldFlag.class, "FieldFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldFlag_Expression(), theCorePackage.getEqualityExpr(), null, "expression", null, 0, 1, FieldFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(enabledFlagEClass, EnabledFlag.class, "EnabledFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(mandatoryFlagEClass, MandatoryFlag.class, "MandatoryFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(visibleFlagEClass, VisibleFlag.class, "VisibleFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(collapsedFlagEClass, CollapsedFlag.class, "CollapsedFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(editableFlagEClass, EditableFlag.class, "EditableFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(labelTextEClass, LabelText.class, "LabelText", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getLabelText_Label(), ecorePackage.getEString(), "label", null, 0, 1, LabelText.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabelText_ReferencedLabelString(), this.getStringDefinition(), null, "referencedLabelString", null, 0, 1, LabelText.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(objectPropertyEClass, ObjectProperty.class, "ObjectProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getObjectProperty_Public(), ecorePackage.getEBoolean(), "public", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_Type(), theCorePackage.getSimpleType(), null, "type", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getObjectProperty_Name(), ecorePackage.getEString(), "name", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_ReadOnly(), theCorePackage.getBoolLiteral(), null, "readOnly", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_Required(), theCorePackage.getBoolLiteral(), null, "required", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(componentEClass, Component.class, "Component", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComponent_XmaComponent(), theGuidesignPackage.getXMAComponent(), null, "xmaComponent", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_StyleProperty(), this.getStyleProperty(), null, "styleProperty", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_ComponentProperties(), this.getObjectProperty(), null, "componentProperties", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Dataobjects(), this.getDataObjectVariable(), null, "dataobjects", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_CustomizedAttributeList(), this.getCustomizeAttributeList(), null, "customizedAttributeList", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Commands(), this.getCommand(), null, "commands", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Events(), this.getEventMappingList(), null, "events", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_ConditionsBlock(), theDomPackage.getConditionsBlock(), null, "conditionsBlock", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Pages(), this.getPage(), null, "pages", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Definitions(), this.getDefinitions(), null, "definitions", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedXMAGuiElementEClass, ReferencedXMAGuiElement.class, "ReferencedXMAGuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(referencedXMAPageEClass, ReferencedXMAPage.class, "ReferencedXMAPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedXMAPage_XmaPage(), theGuidesignPackage.getXMAPage(), null, "xmaPage", null, 0, 1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_GrayLogic(), this.getGrayLogic(), null, "grayLogic", null, 0, 1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedXMACompositeEClass, ReferencedXMAComposite.class, "ReferencedXMAComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedXMAComposite_XmaComposite(), theGuidesignPackage.getXMAComposite(), null, "xmaComposite", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAComposite_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_ReferencingPageComposite(), ecorePackage.getEBoolean(), "referencingPageComposite", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(grayLogicEClass, GrayLogic.class, "GrayLogic", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getGrayLogic_WidgetGrayLogic(), this.getXMAWidgetGrayLogic(), null, "widgetGrayLogic", null, 0, -1, GrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaWidgetGrayLogicEClass, XMAWidgetGrayLogic.class, "XMAWidgetGrayLogic", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXMAWidgetGrayLogic_XmaWidget(), theGuidesignPackage.getXMAWidget(), null, "xmaWidget", null, 0, 1, XMAWidgetGrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXMAWidgetGrayLogic_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, XMAWidgetGrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dataObjectVariableEClass, DataObjectVariable.class, "DataObjectVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataObjectVariable_Type(), theDomPackage.getComplexType(), null, "type", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getDataObjectVariable_IsCollection(), ecorePackage.getEBoolean(), "isCollection", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getDataObjectVariable_Name(), ecorePackage.getEString(), "name", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDataObjectVariable_CustomizedAttributes(), this.getCustomizedAttribute(), null, "customizedAttributes", null, 0, -1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizeAttributeListEClass, CustomizeAttributeList.class, "CustomizeAttributeList", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizeAttributeList_CustomizedAttributes(), this.getCustomizedAttribute(), null, "customizedAttributes", null, 0, -1, CustomizeAttributeList.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizedAttributeEClass, CustomizedAttribute.class, "CustomizedAttribute", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizedAttribute_Field(), this.getIField(), null, "field", null, 0, 1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizedAttribute_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizedAttribute_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizedAttribute_AttributFlag(), theDomPackage.getAttributeFlag(), null, "attributFlag", null, 0, -1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizedAttribute_AttributFlags(), theDomPackage.getAttributeFlag(), null, "attributFlags", null, 0, -1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizedAttribute_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, CustomizedAttribute.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(textPropertiesEClass, TextProperties.class, "TextProperties", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTextProperties_LabelText(), ecorePackage.getEString(), "labelText", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTextProperties_ReferencedLabelString(), this.getStringDefinition(), null, "referencedLabelString", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTextProperties_TooltipText(), ecorePackage.getEString(), "tooltipText", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTextProperties_ReferencedTooltipString(), this.getStringDefinition(), null, "referencedTooltipString", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTextProperties_UnitLabelText(), ecorePackage.getEString(), "unitLabelText", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTextProperties_ReferencedUnitLabelString(), this.getStringDefinition(), null, "referencedUnitLabelString", null, 0, 1, TextProperties.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iFieldEClass, IField.class, "IField", IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getIField_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, IField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIField_Feature(), this.getFieldFeature(), null, "feature", null, 0, 1, IField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldFeatureEClass, FieldFeature.class, "FieldFeature", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldFeature_Feature(), theDomPackage.getPresentableFeature(), null, "feature", null, 0, 1, FieldFeature.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getFieldFeature_SubFeature(), this.getFieldFeature(), null, "subFeature", null, 0, 1, FieldFeature.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldReferenceEClass, FieldReference.class, "FieldReference", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(customizeableFieldEClass, CustomizeableField.class, "CustomizeableField", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCustomizeableField_ControlType(), this.getControlType(), "controlType", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Parts(), this.getFieldPartSpecification(), null, "parts", null, 0, -1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldPartSpecificationEClass, FieldPartSpecification.class, "FieldPartSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getFieldPartSpecification_Part(), this.getFieldPart(), "part", null, 0, 1, FieldPartSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getFieldPartSpecification_LayoutProperties(), this.getLayoutDataProperty(), null, "layoutProperties", null, 0, -1, FieldPartSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(commandEClass, Command.class, "Command", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCommand_Doc(), ecorePackage.getEString(), "doc", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCommand_ClientOnly(), ecorePackage.getEBoolean(), "clientOnly", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCommand_Name(), ecorePackage.getEString(), "name", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventFunctionEClass, EventFunction.class, "EventFunction", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventFunction_Command(), this.getCommand(), null, "command", null, 0, 1, EventFunction.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getEventFunction_PredefinedCommand(), this.getPredefinedCommand(), "predefinedCommand", null, 0, 1, EventFunction.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventMappingListEClass, EventMappingList.class, "EventMappingList", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventMappingList_Mapping(), this.getEventMapping(), null, "mapping", null, 0, -1, EventMappingList.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventMappingEClass, EventMapping.class, "EventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventMapping_EventFunctions(), this.getEventFunction(), null, "eventFunctions", null, 0, -1, EventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(controlEventMappingEClass, ControlEventMapping.class, "ControlEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getControlEventMapping_Event(), this.getEventType(), "event", null, 0, 1, ControlEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaWidgetEventMappingEClass, XMAWidgetEventMapping.class, "XMAWidgetEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXMAWidgetEventMapping_Control(), theGuidesignPackage.getXMAWidget(), null, "control", null, 0, 1, XMAWidgetEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(guiElementEventMappingEClass, GuiElementEventMapping.class, "GuiElementEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getGuiElementEventMapping_Control(), this.getIGuiElementWithEvent(), null, "control", null, 0, 1, GuiElementEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(initEventMappingEClass, InitEventMapping.class, "InitEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(enterEventMappingEClass, EnterEventMapping.class, "EnterEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(leaveEventMappingEClass, LeaveEventMapping.class, "LeaveEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(dataMappingListEClass, DataMappingList.class, "DataMappingList", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataMappingList_Mapping(), this.getDataMapping(), null, "mapping", null, 0, -1, DataMappingList.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dataMappingEClass, DataMapping.class, "DataMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataMapping_Control(), theGuidesignPackage.getIBDAttachable(), null, "control", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDataMapping_BindingElement(), this.getDataBindingElement(), null, "bindingElement", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDataMapping_Field(), this.getFieldReference(), null, "field", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(pageDefinitionEClass, PageDefinition.class, "PageDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(pageEClass, Page.class, "Page", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getPage_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Dataobjects(), this.getDataObjectVariable(), null, "dataobjects", null, 0, -1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_DataMapping(), this.getDataMappingList(), null, "dataMapping", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Commands(), this.getCommand(), null, "commands", null, 0, -1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Events(), this.getEventMappingList(), null, "events", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_ConditionsBlock(), theDomPackage.getConditionsBlock(), null, "conditionsBlock", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Definitions(), this.getDefinitions(), null, "definitions", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmadslPageEClass, XmadslPage.class, "XmadslPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getXmadslPage_Abstract(), ecorePackage.getEBoolean(), "abstract", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslPage_Name(), ecorePackage.getEString(), "name", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_PageProperty(), this.getPageProperty(), null, "pageProperty", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Template(), this.getPage(), null, "template", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_PageProperties(), this.getObjectProperty(), null, "pageProperties", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Extensions(), this.getCustomizationOfGuiElement(), null, "extensions", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(pageCustomizationEClass, PageCustomization.class, "PageCustomization", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getPageCustomization_PageToCustomize(), this.getPage(), null, "pageToCustomize", null, 0, 1, PageCustomization.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPageCustomization_Composites(), this.getComposite(), null, "composites", null, 0, -1, PageCustomization.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(guiElementEClass, GuiElement.class, "GuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(compositeEClass, Composite.class, "Composite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComposite_Content(), this.getContent(), null, "content", null, 0, 1, Composite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(composedElementEClass, ComposedElement.class, "ComposedElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(complexElementEClass, ComplexElement.class, "ComplexElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(simpleElementEClass, SimpleElement.class, "SimpleElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(iDummyEClass, IDummy.class, "IDummy", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIDummy_Name(), ecorePackage.getEString(), "name", null, 0, 1, IDummy.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iGuiElementWithEventEClass, IGuiElementWithEvent.class, "IGuiElementWithEvent", IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(guiElementWithEvent_dummyEClass, GuiElementWithEvent_dummy.class, "GuiElementWithEvent_dummy", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getGuiElementWithEvent_dummy_DummyProperty(), ecorePackage.getEString(), "dummyProperty", null, 0, 1, GuiElementWithEvent_dummy.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(textEClass, Text.class, "Text", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getText_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(comboEClass, Combo.class, "Combo", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCombo_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(checkBoxEClass, CheckBox.class, "CheckBox", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCheckBox_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(listBoxEClass, ListBox.class, "ListBox", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getListBox_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, ListBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(treeEClass, Tree.class, "Tree", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTree_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, Tree.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(radioButtonEClass, RadioButton.class, "RadioButton", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getRadioButton_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_Constraints(), theDomPackage.getConstraint(), null, "constraints", null, 0, -1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_TextProperties(), this.getTextProperties(), null, "textProperties", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(seperatorEClass, Seperator.class, "Seperator", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSeperator_Orientation(), this.getOrientation(), "orientation", null, 0, 1, Seperator.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(labelEClass, Label.class, "Label", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getLabel_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getLabel_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabel_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getLabel_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(imageEClass, Image.class, "Image", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getImage_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Image.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getImage_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Image.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(buttonEClass, Button.class, "Button", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getButton_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getButton_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getButton_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getButton_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iCompositeEClass, IComposite.class, "IComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(xmadslCompositeEClass, XmadslComposite.class, "XmadslComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXmadslComposite_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslComposite_Scrollable(), theCorePackage.getBoolLiteral(), null, "scrollable", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedCompositeEClass, ReferencedComposite.class, "ReferencedComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedComposite_Reference(), this.getXmadslComposite(), null, "reference", null, 0, 1, ReferencedComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableEClass, Table.class, "Table", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTable_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Key(), theDomPackage.getPresentableFeature(), null, "key", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTable_ColumnMinWidth(), ecorePackage.getEInt(), "columnMinWidth", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_MultiSelection(), theCorePackage.getBoolLiteral(), null, "multiSelection", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_CheckSelection(), theCorePackage.getBoolLiteral(), null, "checkSelection", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Columns(), this.getTableColumn(), null, "columns", null, 0, -1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableColumnEClass, TableColumn.class, "TableColumn", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTableColumn_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_AttributeHolder(), theDomPackage.getPresentableFeature(), null, "attributeHolder", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_Visible(), theCorePackage.getBoolLiteral(), null, "visible", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_CustomLabelText(), ecorePackage.getEString(), "customLabelText", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_Align(), this.getContentAlignment(), "align", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_Width(), ecorePackage.getEInt(), "width", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_RelativeWidth(), ecorePackage.getEBoolean(), "relativeWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_MinWidth(), ecorePackage.getEInt(), "minWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_MaxWidth(), ecorePackage.getEInt(), "maxWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iTabFolderEClass, ITabFolder.class, "ITabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getITabFolder_TabItems(), this.getTabPage(), null, "tabItems", null, 0, -1, ITabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabFolderEClass, TabFolder.class, "TabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(referencedTabFolderEClass, ReferencedTabFolder.class, "ReferencedTabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedTabFolder_Reference(), this.getTabFolder(), null, "reference", null, 0, 1, ReferencedTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iTabPageEClass, ITabPage.class, "ITabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(tabPageEClass, TabPage.class, "TabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabPage_Name(), ecorePackage.getEString(), "name", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_Scrollable(), theCorePackage.getBoolLiteral(), null, "scrollable", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_PageProperties(), this.getObjectProperty(), null, "pageProperties", null, 0, -1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedTabPageEClass, ReferencedTabPage.class, "ReferencedTabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedTabPage_Reference(), this.getTabPage(), null, "reference", null, 0, 1, ReferencedTabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(layoutDataPropertyEClass, LayoutDataProperty.class, "LayoutDataProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(pagePropertyEClass, PageProperty.class, "PageProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(heightPropertyEClass, HeightProperty.class, "HeightProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getHeightProperty_Height(), ecorePackage.getEInt(), "height", null, 0, 1, HeightProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(widthPropertyEClass, WidthProperty.class, "WidthProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getWidthProperty_Width(), ecorePackage.getEInt(), "width", null, 0, 1, WidthProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyEClass, StyleProperty.class, "StyleProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStyleProperty_Styles(), theCorePackage.getStyle(), null, "styles", null, 0, -1, StyleProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(titleButtonsPropertyEClass, TitleButtonsProperty.class, "TitleButtonsProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTitleButtonsProperty_Button(), this.getTitleButton(), "button", null, 0, -1, TitleButtonsProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(resizeablePropertyEClass, ResizeableProperty.class, "ResizeableProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getResizeableProperty_Resizeable(), ecorePackage.getEBoolean(), "resizeable", null, 0, 1, ResizeableProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(alignmentPropertyEClass, AlignmentProperty.class, "AlignmentProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getAlignmentProperty_HAlignment(), this.getHorizontalAlignment(), "hAlignment", null, 0, 1, AlignmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getAlignmentProperty_VAlignment(), this.getVerticalAlignment(), "vAlignment", null, 0, 1, AlignmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(composeDataEClass, ComposeData.class, "ComposeData", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComposeData_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_ReferencedTabulator(), this.getTabulatorDefinition(), null, "referencedTabulator", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_CellWidths(), this.getTabulatorPosition(), null, "cellWidths", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getComposeData_CellWidthsCount(), ecorePackage.getEInt(), "cellWidthsCount", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_CellHeights(), this.getTabulatorPosition(), null, "cellHeights", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_Spacing(), this.getPaddingWidth(), null, "spacing", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_Margin(), this.getPaddingWidth(), null, "margin", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(paddingWidthEClass, PaddingWidth.class, "PaddingWidth", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getPaddingWidth_Width(), ecorePackage.getEInt(), "width", null, 0, 1, PaddingWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iAttachmentPositionEClass, IAttachmentPosition.class, "IAttachmentPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIAttachmentPosition_Name(), ecorePackage.getEString(), "name", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_Width(), ecorePackage.getEInt(), "width", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_Relative(), ecorePackage.getEBoolean(), "relative", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_Offset(), this.getOffset(), null, "offset", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_ReferencedPosition(), this.getIAttachmentPosition(), null, "referencedPosition", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_AdditionalOffset(), this.getOffset(), null, "additionalOffset", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorDefinitionEClass, TabulatorDefinition.class, "TabulatorDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, TabulatorDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorDefinition_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, TabulatorDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stringDefinitionEClass, StringDefinition.class, "StringDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStringDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, StringDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStringDefinition_Text(), ecorePackage.getEString(), "text", null, 0, 1, StringDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(integerDefinitionEClass, IntegerDefinition.class, "IntegerDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIntegerDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, IntegerDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIntegerDefinition_Value(), ecorePackage.getEInt(), "value", null, 0, 1, IntegerDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(imageUriEClass, ImageUri.class, "ImageUri", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getImageUri_Name(), ecorePackage.getEString(), "name", null, 0, 1, ImageUri.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getImageUri_Source(), ecorePackage.getEString(), "source", null, 0, 1, ImageUri.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(definitionsEClass, Definitions.class, "Definitions", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDefinitions_Strings(), this.getStringDefinition(), null, "strings", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Integers(), this.getIntegerDefinition(), null, "integers", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_ImageSources(), this.getImageUri(), null, "imageSources", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Positions(), this.getAttachmentPosition(), null, "positions", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Tabulators(), this.getTabulatorDefinition(), null, "tabulators", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorPositionEClass, TabulatorPosition.class, "TabulatorPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorPosition_FloatingPosition(), ecorePackage.getEBoolean(), "floatingPosition", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTabulatorPosition_FillingPosition(), ecorePackage.getEBoolean(), "fillingPosition", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTabulatorPosition_DeltaSpecification(), ecorePackage.getEBoolean(), "deltaSpecification", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentPositionEClass, AttachmentPosition.class, "AttachmentPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(contentEClass, Content.class, "Content", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getContent_WidgetSetItems(), this.getSetOfGuiElements(), null, "widgetSetItems", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_ContainerItems(), this.getComplexElement(), null, "containerItems", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_Elements(), this.getComposedElement(), null, "elements", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(setOfGuiElementsEClass, SetOfGuiElements.class, "SetOfGuiElements", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getSetOfGuiElements_Elements(), this.getComposedElement(), null, "elements", null, 0, -1, SetOfGuiElements.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentPropertyEClass, AttachmentProperty.class, "AttachmentProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getAttachmentProperty_OwnPosition(), this.getAttachmentOwnPosition(), "ownPosition", null, 0, 1, AttachmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getAttachmentProperty_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, AttachmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentSpecificationEClass, AttachmentSpecification.class, "AttachmentSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(noneAttachmentEClass, NoneAttachment.class, "NoneAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getNoneAttachment_None(), ecorePackage.getEBoolean(), "none", null, 0, 1, NoneAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(parentAttachmentEClass, ParentAttachment.class, "ParentAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getParentAttachment_Distance(), ecorePackage.getEInt(), "distance", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getParentAttachment_Relative(), ecorePackage.getEBoolean(), "relative", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getParentAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iElementOnWhichCanBeAttachedEClass, IElementOnWhichCanBeAttached.class, "IElementOnWhichCanBeAttached", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(siblingAttachmentEClass, SiblingAttachment.class, "SiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSiblingAttachment_SiblingPosition(), this.getAttachmentSiblingPosition(), "siblingPosition", null, 0, 1, SiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSiblingAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, SiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(absoluteSiblingAttachmentEClass, AbsoluteSiblingAttachment.class, "AbsoluteSiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getAbsoluteSiblingAttachment_Sibling(), this.getIElementOnWhichCanBeAttached(), null, "sibling", null, 0, 1, AbsoluteSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getAbsoluteSiblingAttachment_FieldPart(), this.getFieldPart(), "fieldPart", null, 0, 1, AbsoluteSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(relativeSiblingAttachmentEClass, RelativeSiblingAttachment.class, "RelativeSiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getRelativeSiblingAttachment_RelativeSibling(), this.getRelativeSibling(), "relativeSibling", null, 0, 1, RelativeSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorAttachmentEClass, TabulatorAttachment.class, "TabulatorAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorAttachment_TabIndex(), ecorePackage.getEInt(), "tabIndex", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorAttachment_Sibling(), this.getIElementOnWhichCanBeAttached(), null, "sibling", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(offsetEClass, Offset.class, "Offset", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getOffset_Negative(), ecorePackage.getEBoolean(), "negative", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getOffset_Value(), ecorePackage.getEInt(), "value", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldVariableEClass, FieldVariable.class, "FieldVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldVariable_Field(), this.getFieldReference(), null, "field", null, 0, 1, FieldVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getFieldVariable_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, FieldVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaVariableEClass, XmaVariable.class, "XmaVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXmaVariable_Reference(), theFlexPackage.getIExpVariable(), null, "reference", null, 0, 1, XmaVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmaVariable_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, XmaVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizeComponentModelEClass, CustomizeComponentModel.class, "CustomizeComponentModel", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizeComponentModel_Imports(), theCorePackage.getImport(), null, "imports", null, 0, -1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeComponentModel_Component(), this.getComponent(), null, "component", null, 0, 1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeComponentModel_Customizations(), this.getCustomizationOfGuiElement(), null, "customizations", null, 0, -1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfGuiElementEClass, CustomizationOfGuiElement.class, "CustomizationOfGuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfGuiElement_Page(), this.getPage(), null, "page", null, 0, 1, CustomizationOfGuiElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfCompositeEClass, CustomizationOfComposite.class, "CustomizationOfComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfComposite_Composite(), this.getIComposite(), null, "composite", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_AddedElements(), this.getComposedElement(), null, "addedElements", null, 0, -1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCustomizationOfComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCustomizationOfComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_AddedWidgetSetItems(), this.getSetOfGuiElements(), null, "addedWidgetSetItems", null, 0, -1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfTabFolderEClass, CustomizationOfTabFolder.class, "CustomizationOfTabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfTabFolder_TabFolder(), this.getITabFolder(), null, "tabFolder", null, 0, 1, CustomizationOfTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfTabFolder_AddedTabItems(), this.getTabPage(), null, "addedTabItems", null, 0, -1, CustomizationOfTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iElementWithLayoutDataEClass, IElementWithLayoutData.class, "IElementWithLayoutData", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getIElementWithLayoutData_LayoutProperties(), this.getLayoutDataProperty(), null, "layoutProperties", null, 0, -1, IElementWithLayoutData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIElementWithLayoutData_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, IElementWithLayoutData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(styleSpecificationEClass, StyleSpecification.class, "StyleSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStyleSpecification_ModelElements(), this.getModelElementSpecification(), null, "modelElements", null, 0, -1, StyleSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStyleSpecification_StyleProperty(), this.getStyleSpecificationProperty(), null, "styleProperty", null, 0, -1, StyleSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(modelElementSpecificationEClass, ModelElementSpecification.class, "ModelElementSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getModelElementSpecification_ModelElement(), this.getSelectedModelElement(), null, "modelElement", null, 0, 1, ModelElementSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getModelElementSpecification_CombinedElements(), this.getCombinedModelElement(), null, "combinedElements", null, 0, -1, ModelElementSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(selectedModelElementEClass, SelectedModelElement.class, "SelectedModelElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSelectedModelElement_Type(), this.getStyleSelectorType(), "type", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_Name(), ecorePackage.getEString(), "name", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_FirstChild(), ecorePackage.getEBoolean(), "firstChild", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_MiddleChild(), ecorePackage.getEBoolean(), "middleChild", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_LastChild(), ecorePackage.getEBoolean(), "lastChild", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(combinedModelElementEClass, CombinedModelElement.class, "CombinedModelElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCombinedModelElement_Child(), ecorePackage.getEBoolean(), "child", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCombinedModelElement_Next(), ecorePackage.getEBoolean(), "next", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombinedModelElement_ModelElement(), this.getSelectedModelElement(), null, "modelElement", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(styleSpecificationPropertyEClass, StyleSpecificationProperty.class, "StyleSpecificationProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(stylePropertyFieldPartEClass, StylePropertyFieldPart.class, "StylePropertyFieldPart", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyFieldPart_Parts(), this.getFieldPartSpecification(), null, "parts", null, 0, -1, StylePropertyFieldPart.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyLeftEClass, StylePropertyLeft.class, "StylePropertyLeft", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyLeft_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyLeft.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyRightEClass, StylePropertyRight.class, "StylePropertyRight", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyRight_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyRight.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTopEClass, StylePropertyTop.class, "StylePropertyTop", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTop_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyTop.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBottomEClass, StylePropertyBottom.class, "StylePropertyBottom", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyBottom_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyBottom.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyWidthEClass, StylePropertyWidth.class, "StylePropertyWidth", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyWidth_Width(), ecorePackage.getEInt(), "width", null, 0, 1, StylePropertyWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyHeightEClass, StylePropertyHeight.class, "StylePropertyHeight", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyHeight_Height(), ecorePackage.getEInt(), "height", null, 0, 1, StylePropertyHeight.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTabulatorEClass, StylePropertyTabulator.class, "StylePropertyTabulator", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTabulator_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, StylePropertyTabulator.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBorderEClass, StylePropertyBorder.class, "StylePropertyBorder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyBorder_NoneBorder(), ecorePackage.getEBoolean(), "noneBorder", null, 0, 1, StylePropertyBorder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyForegroundColorEClass, StylePropertyForegroundColor.class, "StylePropertyForegroundColor", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyForegroundColor_Red(), ecorePackage.getEInt(), "red", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_Blue(), ecorePackage.getEInt(), "blue", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_Green(), ecorePackage.getEInt(), "green", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_ColorID(), ecorePackage.getEString(), "colorID", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBackgroundColorEClass, StylePropertyBackgroundColor.class, "StylePropertyBackgroundColor", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyBackgroundColor_Red(), ecorePackage.getEInt(), "red", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_Blue(), ecorePackage.getEInt(), "blue", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_Green(), ecorePackage.getEInt(), "green", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_ColorID(), ecorePackage.getEString(), "colorID", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyWidgetVariantEClass, StylePropertyWidgetVariant.class, "StylePropertyWidgetVariant", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyWidgetVariant_Name(), ecorePackage.getEString(), "name", null, 0, 1, StylePropertyWidgetVariant.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		// Initialize enums and add enum literals
		initEEnum(eventTypeEEnum, EventType.class, "EventType");
		addEEnumLiteral(eventTypeEEnum, EventType.NULL);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_CLICK);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_DOUBLE_CLICK);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_SELECTION);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_DEFAULT_SELECTION);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_EXPAND);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_COLLAPSE);

		initEEnum(predefinedCommandEEnum, PredefinedCommand.class, "PredefinedCommand");
		addEEnumLiteral(predefinedCommandEEnum, PredefinedCommand.CLOSE_OK);
		addEEnumLiteral(predefinedCommandEEnum, PredefinedCommand.CLOSE_CANCEL);

		initEEnum(attachmentOwnPositionEEnum, AttachmentOwnPosition.class, "AttachmentOwnPosition");
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.LEFT);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.RIGHT);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.TOP);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.BOTTOM);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.VCENTER);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.HCENTER);

		initEEnum(relativeSiblingEEnum, RelativeSibling.class, "RelativeSibling");
		addEEnumLiteral(relativeSiblingEEnum, RelativeSibling.PREVIOUS);
		addEEnumLiteral(relativeSiblingEEnum, RelativeSibling.NEXT);

		initEEnum(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.class, "AttachmentSiblingPosition");
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.NULL);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.LEFT);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.RIGHT);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.TOP);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.BOTTOM);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.CENTER);

		initEEnum(contentAlignmentEEnum, ContentAlignment.class, "ContentAlignment");
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.LEFT);
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.RIGHT);
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.CENTER);

		initEEnum(horizontalAlignmentEEnum, HorizontalAlignment.class, "HorizontalAlignment");
		addEEnumLiteral(horizontalAlignmentEEnum, HorizontalAlignment.LEFT);
		addEEnumLiteral(horizontalAlignmentEEnum, HorizontalAlignment.RIGHT);

		initEEnum(verticalAlignmentEEnum, VerticalAlignment.class, "VerticalAlignment");
		addEEnumLiteral(verticalAlignmentEEnum, VerticalAlignment.TOP);
		addEEnumLiteral(verticalAlignmentEEnum, VerticalAlignment.BOTTOM);

		initEEnum(orientationEEnum, Orientation.class, "Orientation");
		addEEnumLiteral(orientationEEnum, Orientation.HORIZONTAL);
		addEEnumLiteral(orientationEEnum, Orientation.VERTICAL);

		initEEnum(controlTypeEEnum, ControlType.class, "ControlType");
		addEEnumLiteral(controlTypeEEnum, ControlType.TEXT);
		addEEnumLiteral(controlTypeEEnum, ControlType.COMBO);
		addEEnumLiteral(controlTypeEEnum, ControlType.CHECKBOX);
		addEEnumLiteral(controlTypeEEnum, ControlType.LISTBOX);

		initEEnum(fieldPartEEnum, FieldPart.class, "FieldPart");
		addEEnumLiteral(fieldPartEEnum, FieldPart.LABEL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.CONTROL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.UNITLABEL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.EDITOR);

		initEEnum(titleButtonEEnum, TitleButton.class, "TitleButton");
		addEEnumLiteral(titleButtonEEnum, TitleButton.MINIMIZE);
		addEEnumLiteral(titleButtonEEnum, TitleButton.MAXIMIZE);
		addEEnumLiteral(titleButtonEEnum, TitleButton.CLOSE);

		initEEnum(styleSelectorTypeEEnum, StyleSelectorType.class, "StyleSelectorType");
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.LABEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.FIELD);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.CELL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TEXT);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.COMBO);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABLE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TREE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.CHECKBOX);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.RADIOBUTTON);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.LISTBOX);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.PANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.VPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.HPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABFOLDER);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABPAGE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.IMAGE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.SEPERATOR);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.BUTTON);

		// Create resource
		createResource(eNS_URI);
	}

} //PomPackageImpl
